@@ -4,16 +4,18 @@ module Distribution.ShellHarness ( runTests ) where
 import Prelude hiding( catch )
 import System.Directory ( getCurrentDirectory, setPermissions,
                           Permissions(..), getDirectoryContents,
-                          findExecutable, createDirectoryIfMissing )
+                          findExecutable, createDirectoryIfMissing,
+                          renameFile, removeFile )
 import System.Environment ( getEnv, getEnvironment )
 import System.Exit ( ExitCode (..) )
 import System.FilePath
 import System.IO
+import System( system )
 import System.Process ( ProcessHandle,
                         runInteractiveProcess, waitForProcess,
                         getProcessExitCode )
 import Data.Maybe
-import Data.List ( isInfixOf, isPrefixOf, (\\), nubBy )
+import Data.List ( isInfixOf, isPrefixOf, (\\), nubBy, isSuffixOf )
 import Control.Concurrent
 #if __GLASGOW_HASKELL__ >= 610
 import Control.OldException
@@ -40,12 +42,15 @@ runTests darcs_path cwd tests = do
                          return True
 
 run :: Maybe FilePath -> [String] -> IO [String]
-run set_darcs_path tests= do
+run set_darcs_path tests = do
     cwd <-  getCurrentDirectory
     path <- getEnv "PATH"
     env <- getEnvironment
+    bash <- find_bash
     darcs_path <- case set_darcs_path of
-                    Nothing -> get_darcs_path
+                    Nothing -> case lookup "DARCS" env of
+                                 Nothing -> return (cwd ++ "/..")
+                                 Just d  -> return $ takeDirectory d
                     Just x -> return x
     let myenv = [("HOME",cwd)
                 ,("TESTS_WD",cwd)
@@ -55,40 +60,48 @@ run set_darcs_path tests= do
                 ,("PATH",(darcs_path++":"++path))
                 ,("DARCS_DONT_COLOR","1")
                 ,("DARCS_DONT_ESCAPE_ANYTHING","1")]
-    bash <- find_bash
-    let shell = takeWhile (/= '\n') bash
+        shell = takeWhile (/= '\n') bash
     putStrLn $ "Using bash shell in '"++shell++"'"
-    set_prefs
+    catch (appendFile (".darcs/defaults") "\nALL --ignore-times\n")
+          (\e -> fail $ "Unable to set preferences: " ++ show e)
     run_helper shell tests []  (set_env myenv env)
-  where get_darcs_path = do
-           env <- getEnvironment
-           cwd <-  getCurrentDirectory
-           case lookup "DARCS" env of
-                Nothing -> return (cwd ++ "/..")
-                Just d  -> return $ takeDirectory d
-        set_prefs = do
-            finally (catch (appendFile ".darcs/defaults" "\nALL --ignore-times\n")
-                           (\e -> fail $ "Unable to set preferences: "
-                                         ++ show e))
-                    (createDirectoryIfMissing  False ".darcs")
+
+data Status = Success | Failed | Skipped
+
 run_helper :: String -> [String] -> [String] ->
                   [(String,String)] -> IO [String]
 run_helper _ [] fails _ = return fails
 run_helper shell (test:ts) fails env = do
     putStr $ "Running " ++ test ++ " ..." ++ (replicate (36 - (length test)) ' ')
-    (output,success) <- backtick shell test env
-    if success then do putStrLn " passed."
-                       cleanup_dirs
-                       run_helper shell ts fails env
-               else do putStrLn " failed."
-                       putStrLn $ "Probable reason :" ++ output
-                       cleanup_dirs
-                       run_helper shell ts (fails++[test]) env
-  where cleanup_dirs :: IO ()
-        cleanup_dirs =
+    (output,result) <- backtick shell test env
+    cleanup
+    case result of
+      Skipped -> do putStrLn " skipped."
+                    run_helper shell ts fails env
+      Success -> do putStrLn " passed."
+                    run_helper shell ts fails env
+      Failed -> do putStrLn " failed."
+                   putStrLn $ "Probable reason :" ++ output
+                   run_helper shell ts (fails++[test]) env
+  where cleanup :: IO ()
+        cleanup =
           do dirfiles <- getDirectoryContents (fromJust $ lookup "TESTS_WD" env)
              let tempfiles = (filter ("temp" `isPrefixOf`) dirfiles) ++
                              (filter ("tmp" `isPrefixOf`) dirfiles)
+             when (isJust $ lookup "HPCTIXDIR" env) $ do
+                 let tixdir = fromJust $ lookup "HPCTIXDIR" env
+                 tixlist <- getDirectoryContents tixdir
+                 oldsum <- if ("sum.tix" `elem` tixlist)
+                              then do renameFile (tixdir </> "sum.tix")
+                                                 (tixdir </> "oldsum.tix")
+                                      return [tixdir </> "oldsum.tix"]
+                              else return []
+                 let tixfiles = oldsum ++ [ tixdir </> f | f <- tixlist
+                                                         , "darcs-" `isPrefixOf` f
+                                                         , ".tix" `isSuffixOf` f ]
+                 system $ "hpc sum --union --output=" ++ tixdir </> "sum.tix" ++ " " ++ unwords tixfiles
+                 forM tixfiles $ \f -> removeFile f
+                 return ()
              mapM_ (\x-> 
                   setPermissions x (Permissions 
                                    {readable = True
@@ -98,12 +111,13 @@ run_helper shell (test:ts) fails env = do
                                    )
                  ) tempfiles
 
-backtick :: String -> String -> [(String, String)]-> IO (String,Bool)
+backtick :: String -> String -> [(String, String)]-> IO (String,Status)
 backtick cmd args env = do
    (exitcode,res) <- backtick_helper cmd args env
    case exitcode of
-        ExitSuccess -> return (res, True)
-        ExitFailure _ -> return (res, False)
+        ExitSuccess -> return (res, Success)
+        ExitFailure 200 -> return (res, Skipped)
+        ExitFailure _ -> return (res, Failed)
 
 backtick_helper :: String -> String -> [(String,String)] ->
                                       IO (ExitCode, String)
