@@ -18,7 +18,6 @@
 module ByteStringUtils (
 
         unsafeWithInternals,
-        unpackPSfromUTF8,
 
         -- IO with mmap or gzip
         gzReadFilePS,
@@ -28,9 +27,7 @@ module ByteStringUtils (
 
         -- gzip handling
         isGZFile,
-#ifdef HAVE_HASKELL_ZLIB
         gzDecompress,
-#endif
 
         -- list utilities
         ifHeadThenTail,
@@ -59,27 +56,22 @@ import qualified Data.ByteString.Internal   as BI
 import Data.ByteString (intercalate, uncons)
 import Data.ByteString.Internal (fromForeignPtr)
 
-#if defined (HAVE_MMAP) || ! defined (HAVE_HASKELL_ZLIB)
+#if defined (HAVE_MMAP)
 import Control.Exception        ( catch )
 #endif
 import System.IO
 import System.IO.Unsafe         ( unsafePerformIO )
 
 import Foreign.Storable         ( peekElemOff, peek )
-import Foreign.Marshal.Alloc    ( free )
-import Foreign.Marshal.Array    ( mallocArray, peekArray, advancePtr )
+import Foreign.Marshal.Array    ( advancePtr )
 import Foreign.C.Types          ( CInt )
 
 import Data.Bits                ( rotateL )
-import Data.Char                ( chr, ord, isSpace )
+import Data.Char                ( ord, isSpace )
 import Data.Word                ( Word8 )
 import Data.Int                 ( Int32 )
 import Control.Monad            ( when )
 
-#ifndef HAVE_HASKELL_ZLIB
-import Foreign.Ptr              ( nullPtr )
-import Foreign.ForeignPtr       ( ForeignPtr )
-#endif
 import Foreign.Ptr              ( plusPtr, Ptr )
 import Foreign.ForeignPtr       ( withForeignPtr )
 
@@ -88,14 +80,10 @@ import Foreign.ForeignPtr       ( addForeignPtrFinalizer )
 import Foreign.Ptr              ( FunPtr )
 #endif
 
-#if HAVE_HASKELL_ZLIB
 import qualified Data.ByteString.Lazy as BL
 import qualified Codec.Compression.GZip as GZ
 import qualified Codec.Compression.Zlib.Internal as ZI
 import Darcs.Global ( addCRCWarning )
-#else
-import Foreign.C.String ( CString, withCString )
-#endif
 
 #ifdef HAVE_MMAP
 import System.IO.MMap( mmapFileByteString )
@@ -106,22 +94,6 @@ import System.Posix.Files( fileSize, getSymbolicLinkStatus )
 -- -----------------------------------------------------------------------------
 -- obsolete debugging code
 
-# ifndef HAVE_HASKELL_ZLIB
-debugForeignPtr :: ForeignPtr a -> String -> IO ()
-#ifdef DEBUG_PS
-foreign import ccall unsafe "static fpstring.h debug_alloc" debug_alloc
-    :: Ptr a -> CString -> IO ()
-foreign import ccall unsafe "static fpstring.h & debug_free" debug_free
-    :: FunPtr (Ptr a -> IO ())
-debugForeignPtr fp n =
-    withCString n $ \cname-> withForeignPtr fp $ \p->
-    do debug_alloc p cname
-       addForeignPtrFinalizer debug_free fp
-#else
-debugForeignPtr _ _ = return ()
-#endif
-#endif
-
 -- -----------------------------------------------------------------------------
 -- unsafeWithInternals
 
@@ -141,26 +113,6 @@ unsafeWithInternals ps f
 readIntPS :: B.ByteString -> Maybe (Int, B.ByteString)
 readIntPS = BC.readInt . BC.dropWhile isSpace
 
--- -----------------------------------------------------------------------------
--- Destructor functions (taking PackedStrings apart)
-
-unpackPSfromUTF8 :: B.ByteString -> String
-unpackPSfromUTF8 ps =
- case BI.toForeignPtr ps of
-   (_,_, 0) -> ""
-   (x,s,l)  ->
-    unsafePerformIO $ withForeignPtr x $ \p->
-    do outbuf <- mallocArray l
-       lout <- fromIntegral `fmap`
-               utf8_to_ints outbuf (p `plusPtr` s) (fromIntegral l)
-       when (lout < 0) $ error "Bad UTF8!"
-       str <- (map (chr . fromIntegral)) `fmap` peekArray lout outbuf
-       free outbuf
-       return str
-
-foreign import ccall unsafe "static fpstring.h utf8_to_ints" utf8_to_ints
-    :: Ptr Int -> Ptr Word8 -> CInt -> IO CInt
-
 -- -----------------------------------------------------------------------------
 -- List-mimicking functions for PackedStrings
 
@@ -343,8 +295,6 @@ unlinesPSOld ss = BC.concat $ intersperse_newlines ss
 -- -----------------------------------------------------------------------------
 -- gzReadFilePS
 
-#ifdef HAVE_HASKELL_ZLIB
-
 -- |Decompress the given bytestring into a lazy list of chunks, along with a boolean
 -- flag indicating (if True) that the CRC was corrupted.
 -- Inspecting the flag will cause the entire list of chunks to be evaluated (but if
@@ -353,7 +303,7 @@ gzDecompress :: Maybe Int -> BL.ByteString -> ([B.ByteString], Bool)
 gzDecompress mbufsize =
     -- This is what the code would be without the bad CRC recovery logic:
     -- return . BL.toChunks . GZ.decompressWith decompressParams
-    toListWarn . ZI.decompressWithErrors ZI.GZip decompressParams
+    toListWarn . ZI.decompressWithErrors ZI.gzipFormat decompressParams
   where
         decompressParams = case mbufsize of
                               Just bufsize -> GZ.defaultDecompressParams { GZ.decompressBufferSize = bufsize }
@@ -378,19 +328,6 @@ gzDecompress mbufsize =
         handleBad ZI.DataError "incorrect data check" = ([], True)
         handleBad _ msg = error msg
 
-#else
-
-foreign import ccall unsafe "static zlib.h gzopen" c_gzopen
-    :: CString -> CString -> IO (Ptr ())
-foreign import ccall unsafe "static zlib.h gzclose" c_gzclose
-    :: Ptr () -> IO ()
-foreign import ccall unsafe "static zlib.h gzread" c_gzread
-    :: Ptr () -> Ptr Word8 -> CInt -> IO CInt
-foreign import ccall unsafe "static zlib.h gzwrite" c_gzwrite
-    :: Ptr () -> Ptr Word8 -> CInt -> IO CInt
-
-#endif
-
 isGZFile :: FilePath -> IO (Maybe Int)
 isGZFile f = do
     h <- openBinaryFile f ReadMode
@@ -411,7 +348,6 @@ gzReadFilePS f = do
     case mlen of
        Nothing -> mmapFilePS f
        Just len ->
-#ifdef HAVE_HASKELL_ZLIB
             do -- Passing the length to gzDecompress means that it produces produces one chunk,
                -- which in turn means that B.concat won't need to copy data.
                -- If the length is wrong this will just affect efficiency, not correctness
@@ -420,19 +356,6 @@ gzReadFilePS f = do
                                             return res
                compressed <- (BL.fromChunks . return) `fmap` mmapFilePS f
                B.concat `fmap` doDecompress compressed
-#else
-               withCString f $ \fstr-> withCString "rb" $ \rb-> do
-                 gzf <- c_gzopen fstr rb
-                 when (gzf == nullPtr) $ fail $ "problem opening file "++f
-                 fp <- BI.mallocByteString len
-                 debugForeignPtr fp $ "gzReadFilePS "++f
-                 lread <- withForeignPtr fp $ \p ->
-                          c_gzread gzf p (fromIntegral len)
-                 c_gzclose gzf
-                 when (fromIntegral lread /= len) $
-                      fail $ "problem gzreading file "++f
-                 return $ fromForeignPtr fp 0 len
-#endif
 
 hGetLittleEndInt :: Handle -> IO Int
 hGetLittleEndInt h = do
@@ -447,26 +370,7 @@ gzWriteFilePS f ps = gzWriteFilePSs f [ps]
 
 gzWriteFilePSs :: FilePath -> [B.ByteString] -> IO ()
 gzWriteFilePSs f pss  =
-#ifdef HAVE_HASKELL_ZLIB
     BL.writeFile f $ GZ.compress $ BL.fromChunks pss
-#else
-    withCString f $ \fstr -> withCString "wb" $ \wb -> do
-    gzf <- c_gzopen fstr wb
-    when (gzf == nullPtr) $ fail $ "problem gzopening file for write: "++f
-    mapM_ (gzWriteToGzf gzf) pss `catch`
-              \_ -> fail $ "problem gzwriting file: "++f
-    c_gzclose gzf
-
-gzWriteToGzf :: Ptr () -> B.ByteString -> IO ()
-gzWriteToGzf gzf ps = case BI.toForeignPtr ps of
- (_,_,0) -> return () -- avoid calling gzwrite with 0 length this would
-                      -- trouble on some versions of zlib, and is always
-                      -- unnecessary.
- (x,s,l) -> do
-    lw <- withForeignPtr x $ \p -> c_gzwrite gzf (p `plusPtr` s)
-                                                 (fromIntegral l)
-    when (fromIntegral lw /= l) $ fail $ "problem in gzWriteToGzf"
-#endif
 
 -- -----------------------------------------------------------------------------
 -- mmapFilePS
