@@ -10,17 +10,19 @@ module Darcs.External (
     execDocPipe, execPipeIgnoreError,
     getTermNColors,
     pipeDoc, pipeDocSSH, execSSH,
+    remoteDarcsCmd,
     maybeURLCmd,
     Cachable(Cachable, Uncachable, MaxAge),
     viewDoc, viewDocWith,
-    sendmail_path, diff_program
+    sendmail_path, diff_program, darcs_program
   ) where
 
+import qualified Ratified
 import Data.Maybe ( isJust, isNothing, maybeToList )
 import Control.Monad ( when, zipWithM_, filterM, liftM2 )
 import System.Exit ( ExitCode(..) )
-import System.Environment ( getEnv )
-import System.IO ( hPutStr, hPutStrLn, hGetContents, hClose,
+import System.Environment ( getEnv, getProgName )
+import System.IO ( hSetBinaryMode, hPutStr, hPutStrLn, hClose,
                    openBinaryFile, IOMode( ReadMode ),
                    openBinaryTempFile,
                    hIsTerminalDevice, stdout, stderr, Handle )
@@ -49,14 +51,14 @@ import System.Directory ( createDirectoryIfMissing )
 import System.FilePath.Posix ( (</>), takeDirectory, normalise )
 
 import Darcs.Flags ( DarcsFlag( SignAs, Sign, SignSSL, NoLinks,
-                                Verify, VerifySSL ) )
+                                Verify, VerifySSL, RemoteDarcs ) )
 import Darcs.RepoPath ( AbsolutePath, toFilePath )
 import Darcs.Utils ( withCurrentDirectory, breakCommand, get_viewer, ortryrunning, )
 import Progress ( withoutProgress, progressList, debugMessage )
 
 import ByteStringUtils (gzReadFilePS, linesPS, unlinesPS)
-import qualified Data.ByteString as B (ByteString, empty, null, readFile -- ratify readFile: Just an import from ByteString
-            ,hGetContents, writeFile, hPut, length -- ratify hGetContents: importing from ByteString
+import qualified Data.ByteString as B (ByteString, empty, null, readFile
+            ,hGetContents, writeFile, hPut, length
             ,take, concat, drop, isPrefixOf, singleton, append)
 import qualified Data.ByteString.Char8 as BC (unpack, pack)
 
@@ -88,6 +90,11 @@ diff_program = do
   when (null l) $ fail "Cannot find the \"diff\" program."
   return $ head l
 
+-- |Get the name of the darcs executable (as supplied by @getProgName@)
+darcs_program :: IO String
+darcs_program = getProgName
+-- Another option: getEnv "DARCS" `catch` \_ -> getProgName
+
 backupByRenaming :: FilePath -> IO ()
 backupByRenaming = backupBy rename
  where rename x y = do
@@ -123,19 +130,23 @@ backupBy backup f =
 -- will use a cache as required by its second argument.
 fetchFilePS :: String -> Cachable -> IO B.ByteString
 fetchFilePS fou _ | is_file fou = B.readFile fou
-fetchFilePS fou cache = withTemp $ \t -> do copyFileOrUrl [] fou t cache
+fetchFilePS fou cache = withTemp $ \t -> do let opts = [] -- FIXME: no network flags
+                                            copyFileOrUrl opts fou t cache
                                             B.readFile t
 
 gzFetchFilePS :: String -> Cachable -> IO B.ByteString
 gzFetchFilePS fou _ | is_file fou = gzReadFilePS fou
-gzFetchFilePS fou cache = withTemp $ \t-> do copyFileOrUrl [] fou t cache
+gzFetchFilePS fou cache = withTemp $ \t-> do let opts = [] -- FIXME: no network flags
+                                             copyFileOrUrl opts fou t cache
                                              gzReadFilePS t
 
+remoteDarcsCmd :: [DarcsFlag] -> String
+remoteDarcsCmd flags = head $ [ c | (RemoteDarcs c) <- flags ] ++ ["darcs"]
 
 copyFileOrUrl :: [DarcsFlag] -> FilePath -> FilePath -> Cachable -> IO ()
 copyFileOrUrl opts fou out _     | is_file fou = copyLocal opts fou out
 copyFileOrUrl _    fou out cache | is_url  fou = copyRemote fou out cache
-copyFileOrUrl _    fou out _     | is_ssh  fou = copySSH fou out
+copyFileOrUrl opts fou out _     | is_ssh  fou = copySSH (remoteDarcsCmd opts) fou out
 copyFileOrUrl _    fou _   _     = fail $ "unknown transport protocol: " ++ fou
 
 speculateFileOrUrl :: String -> FilePath -> IO ()
@@ -240,7 +251,7 @@ copyRemoteNormal u v cache = copyUrlFirst u v cache >> waitUrl u
 copyFilesOrUrls :: [DarcsFlag]->FilePath->[String]->FilePath->Cachable->IO ()
 copyFilesOrUrls opts dou ns out _ | is_file dou = copyLocals opts dou ns out
 copyFilesOrUrls _ dou ns out c    | is_url  dou = copyRemotes dou ns out c
-copyFilesOrUrls _ dou ns out _    | is_ssh  dou = copySSHs dou ns out
+copyFilesOrUrls opts dou ns out _ | is_ssh  dou = copySSHs (remoteDarcsCmd opts) dou ns out
 copyFilesOrUrls _ dou _  _   _    = fail $ "unknown transport protocol: "++dou
 
 
@@ -305,12 +316,14 @@ pipeDoc :: String -> [String] -> Doc -> IO ExitCode
 pipeDoc c args inp = withoutNonBlock $ withoutProgress $
     do debugMessage $ unwords (c:args)
        (i,o,e,pid) <- runInteractiveProcess c args Nothing Nothing
+       hSetBinaryMode i True
+       hSetBinaryMode o True
        mvare <- newEmptyMVar
-       forkIO ((hGetContents e >>= -- ratify hGetContents: it's immediately consumed
+       forkIO ((Ratified.hGetContents e >>= -- ratify: immediately consumed
                 hPutStr stderr)
                `finally` putMVar mvare ())
        mvaro <- newEmptyMVar
-       forkIO ((hGetContents o >>= -- ratify hGetContents: it's immediately consumed
+       forkIO ((Ratified.hGetContents o >>= -- ratify: immediately consumed
                 hPutStr stdout)
                `finally` putMVar mvaro ())
        hPutDoc i inp
@@ -485,7 +498,7 @@ execDocPipe c args instr = withoutProgress $
     do (i,o,e,pid) <- runInteractiveProcess c args Nothing Nothing
        forkIO $ hPutDoc i instr >> hClose i
        mvare <- newEmptyMVar
-       forkIO ((hGetContents e >>= -- ratify hGetContents: it's immediately consumed
+       forkIO ((Ratified.hGetContents e >>= -- ratify: immediately consumed
                 hPutStr stderr)
                `finally` putMVar mvare ())
        out <- B.hGetContents o
@@ -503,7 +516,7 @@ execPipeIgnoreError c args instr = withoutProgress $
     do (i,o,e,pid) <- runInteractiveProcess c args Nothing Nothing
        forkIO $ hPutDoc i instr >> hClose i
        mvare <- newEmptyMVar
-       forkIO ((hGetContents e >>= -- ratify hGetContents: it's immediately consumed
+       forkIO ((Ratified.hGetContents e >>= -- ratify: immediately consumed
                 hPutStr stderr)
                `finally` putMVar mvare ())
        out <- B.hGetContents o
@@ -620,8 +633,9 @@ viewDocWith :: Printers -> Doc -> IO ()
 viewDocWith pr msg = do
   isTerminal <- hIsTerminalDevice stdout
   if isTerminal && lengthGreaterThan (20 :: Int) (lines $ renderString msg)
-     then do viewer <- get_viewer
-             pipeDocToPager viewer [] pr msg
+     then do viewerPlusArgs <- get_viewer
+             let (viewer:args) = words viewerPlusArgs
+             pipeDocToPager viewer args pr msg
                `ortryrunning` pipeDocToPager  "less" [] pr msg
                `ortryrunning` pipeDocToPager  "more" [] pr msg
 #ifdef WIN32
