@@ -15,16 +15,21 @@
 -- the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 -- Boston, MA 02110-1301, USA.
 
-module Darcs.Flags ( DarcsFlag( .. ), Compression( .. ), compression, want_external_merge, isInteractive,
-                     maxCount,
+module Darcs.Flags ( DarcsFlag( .. ), Compression( .. ), compression, 
+                     want_external_merge, isInteractive,
+                     maxCount, willIgnoreTimes, willRemoveLogFile, isUnified,
+                     willStoreInMemory, doHappyForwarding, includeBoring,
+                     doAllowCaseOnly, doAllowWindowsReserved, doReverse,
+                     showChangesOnlyToFiles
                    ) where
+import Data.Maybe( fromMaybe )
 import Darcs.Patch.MatchData ( PatchMatch )
 import Darcs.RepoPath ( AbsolutePath, AbsolutePathOrStd )
 
 -- | The 'DarcsFlag' type is a list of all flags that can ever be
 -- passed to darcs, or to one of its commands.
 data DarcsFlag = Help | ListOptions | NoTest | Test
-               | OnlyChangesToFiles
+               | OnlyChangesToFiles | ChangesToAllFiles
                | LeaveTestDir | NoLeaveTestDir
                | Timings | Debug | DebugVerbose | DebugHTTP
                | Verbose | NormalVerbosity | Quiet
@@ -38,22 +43,27 @@ data DarcsFlag = Help | ListOptions | NoTest | Test
                | NumberPatches
                | OneTag String | AfterTag String | UpToTag String
                | Context AbsolutePath | Count
-               | LogFile AbsolutePath | RmLogFile
+               | LogFile AbsolutePath | RmLogFile | DontRmLogFile
                | DistName String | All
                | Recursive | NoRecursive | Reorder
                | RestrictPaths | DontRestrictPaths
-               | AskDeps | NoAskDeps | IgnoreTimes | LookForAdds | NoLookForAdds
+               | AskDeps | NoAskDeps | IgnoreTimes | DontIgnoreTimes
+               | LookForAdds | NoLookForAdds
                | AnyOrder | CreatorHash String
                | Intersection | Union | Complement
                | Sign | SignAs String | NoSign | SignSSL String
-               | HappyForwarding
+               | HappyForwarding | NoHappyForwarding
                | Verify AbsolutePath | VerifySSL AbsolutePath
                | SSHControlMaster | NoSSHControlMaster
+               | RemoteDarcs String
                | EditDescription | NoEditDescription
                | Toks String
                | EditLongComment | NoEditLongComment | PromptLongComment
                | AllowConflicts | MarkConflicts | NoAllowConflicts
-               | Boring | AllowCaseOnly | AllowWindowsReserved
+               | SkipConflicts
+               | Boring | SkipBoring
+               | AllowCaseOnly | DontAllowCaseOnly
+               | AllowWindowsReserved | DontAllowWindowsReserved
                | DontGrabDeps | DontPromptForDependencies | PromptForDependencies
                | Compress | NoCompress | UnCompress
                | WorkRepoDir String | WorkRepoUrl String | RemoteRepo String
@@ -63,8 +73,8 @@ data DarcsFlag = Help | ListOptions | NoTest | Test
                | Pipe | Interactive
                | DiffCmd String
                | ExternalMerge String | Summary | NoSummary
-               | Unified | Reverse
-               | CheckPoint | Partial | Complete | Lazy | Ephemeral
+               | Unified | NonUnified | Reverse | Forward
+               | Partial | Complete | Lazy | Ephemeral
                | FixFilePath AbsolutePath AbsolutePath | DiffFlags String
                | XMLOutput
                | ForceReplace
@@ -78,12 +88,14 @@ data DarcsFlag = Help | ListOptions | NoTest | Test
                | UseFormat2
                | PristinePlain | PristineNone | NoUpdateWorking
                | Sibling AbsolutePath | Relink | RelinkPristine | NoLinks
+               | OptimizePristine
+               | UpgradeFormat
                | Files | NoFiles | Directories | NoDirectories
                | Pending | NoPending
                | PosthookCmd String | NoPosthook | AskPosthook | RunPosthook
                | PrehookCmd String  | NoPrehook  | AskPrehook  | RunPrehook
                | UMask String
-               | StoreInMemory
+               | StoreInMemory | ApplyOnDisk
                | HTTPPipelining | NoHTTPPipelining
                | NoCache
                | AllowUnrelatedRepos
@@ -114,3 +126,43 @@ maxCount :: [DarcsFlag] -> Maybe Int
 maxCount (MaxCount n : _) = Just n
 maxCount (_:xs) = maxCount xs
 maxCount [] = Nothing
+
+-- | @lastWord [(flag, value)] default opts@ scans @opts@ for a flag
+-- in the list and returns the value of the first match, or @default@
+-- if none is found.
+lastWord :: [(DarcsFlag,a)] -> a -> [DarcsFlag] -> a
+lastWord known_flags = foldr . flip $ \ def -> fromMaybe def . flip lookup known_flags
+
+getBoolFlag :: DarcsFlag -> DarcsFlag -> [DarcsFlag] -> Bool
+getBoolFlag t f = lastWord [(t, True), (f, False)] False
+
+willIgnoreTimes :: [DarcsFlag] -> Bool
+willIgnoreTimes = getBoolFlag IgnoreTimes DontIgnoreTimes
+
+willRemoveLogFile :: [DarcsFlag] -> Bool
+willRemoveLogFile = getBoolFlag RmLogFile DontRmLogFile
+
+isUnified :: [DarcsFlag] -> Bool
+isUnified = getBoolFlag Unified NonUnified
+
+willStoreInMemory :: [DarcsFlag] -> Bool
+willStoreInMemory = getBoolFlag Unified NonUnified
+
+doHappyForwarding :: [DarcsFlag] -> Bool
+doHappyForwarding = getBoolFlag HappyForwarding NoHappyForwarding
+
+includeBoring :: [DarcsFlag] -> Bool
+includeBoring = getBoolFlag Boring SkipBoring
+
+doAllowCaseOnly :: [DarcsFlag] -> Bool
+doAllowCaseOnly = getBoolFlag AllowCaseOnly DontAllowCaseOnly
+
+
+doAllowWindowsReserved :: [DarcsFlag] -> Bool
+doAllowWindowsReserved = getBoolFlag AllowWindowsReserved DontAllowWindowsReserved
+
+doReverse :: [DarcsFlag] -> Bool
+doReverse = getBoolFlag Reverse Forward
+
+showChangesOnlyToFiles :: [DarcsFlag] -> Bool
+showChangesOnlyToFiles = getBoolFlag OnlyChangesToFiles ChangesToAllFiles
\ No newline at end of file
