@@ -26,9 +26,7 @@ module Darcs.Global ( atexit, with_atexit,
                       timingsMode, setTimingsMode,
                       whenDebugMode, withDebugMode, setDebugMode,
                       debugMessage, debugFail, putTiming,
-#ifdef HAVE_HASKELL_ZLIB
                       addCRCWarning, getCRCWarnings, resetCRCWarnings,
-#endif
                       darcsdir
     ) where
 
@@ -36,9 +34,7 @@ import Control.Monad ( when )
 import Control.Concurrent.MVar
 import Control.Exception (bracket_, catch, block, unblock)
 import Data.IORef ( IORef, newIORef, readIORef, writeIORef )
-#ifdef HAVE_HASKELL_ZLIB
 import Data.IORef ( modifyIORef )
-#endif
 import System.IO.Unsafe (unsafePerformIO)
 import System.IO (hPutStrLn, hPutStr, stderr)
 import System.Time ( calendarTimeToString, toCalendarTime, getClockTime )
@@ -139,7 +135,6 @@ setSshControlMasterDisabled = writeIORef _sshControlMasterDisabled True
 sshControlMasterDisabled :: Bool
 sshControlMasterDisabled = unsafePerformIO $ readIORef _sshControlMasterDisabled
 
-#ifdef HAVE_HASKELL_ZLIB
 type CRCWarningList = [FilePath]
 {-# NOINLINE _crcWarningList #-}
 _crcWarningList :: IORef CRCWarningList
@@ -153,7 +148,6 @@ getCRCWarnings = readIORef _crcWarningList
 
 resetCRCWarnings :: IO ()
 resetCRCWarnings = writeIORef _crcWarningList []
-#endif
 
 darcsdir :: String
 darcsdir = "_darcs"
