@@ -24,31 +24,33 @@ module Darcs.Patch.Bundle ( hash_bundle, make_bundle, make_bundle2, scan_bundle,
                      make_context, scan_context,
                    ) where
 
-import Darcs.Flags ( DarcsFlag( Unified ) )
+import Darcs.Flags ( DarcsFlag, isUnified )
 import Darcs.Hopefully ( PatchInfoAnd, piap,
                          patchInfoAndPatch,
                          unavailable, hopefully )
 import Darcs.Patch ( RepoPatch, Named, showPatch, showContextPatch, readPatch )
 import Darcs.Patch.Info ( PatchInfo, readPatchInfo, showPatchInfo, human_friendly, is_tag )
 import Darcs.Patch.Set ( PatchSet, SealedPatchSet )
-import Darcs.Ordered ( RL(..), FL(..), unsafeCoerceP,
+import Darcs.Witnesses.Ordered ( RL(..), FL(..), unsafeCoerceP,
                              reverseFL, (+<+), mapFL, mapFL_FL )
 import Printer ( Doc, renderPS, newline, text, ($$),
                  (<>), vcat, vsep, renderString )
-import Darcs.SlurpDirectory ( Slurpy )
 
 import ByteStringUtils ( linesPS, unlinesPS, dropSpace, substrPS)
 import qualified Data.ByteString as B (ByteString, length, null, drop, isPrefixOf)
 import qualified Data.ByteString.Char8 as BC (unpack, break, pack)
 
 import SHA1( sha1PS )
-import Darcs.Sealed ( Sealed(Sealed), mapSeal )
+import Darcs.Witnesses.Sealed ( Sealed(Sealed), mapSeal )
+
+import Storage.Hashed.Tree( Tree )
+import Storage.Hashed.Monad( virtualTreeIO )
 
 hash_bundle :: RepoPatch p => [PatchInfo] -> FL (Named p) C(x y) -> String
 hash_bundle _ to_be_sent = sha1PS $ renderPS
                          $ vcat (mapFL showPatch to_be_sent) <> newline
 
-make_bundle :: RepoPatch p => [DarcsFlag] -> Slurpy -> [PatchInfo] -> FL (Named p) C(x y) -> Doc
+make_bundle :: RepoPatch p => [DarcsFlag] -> Tree IO -> [PatchInfo] -> FL (Named p) C(x y) -> IO Doc
 make_bundle opts the_s common to_be_sent = make_bundle2 opts the_s common to_be_sent to_be_sent
 
 -- | In make_bundle2, it is presumed that the two patch sequences are
@@ -56,23 +58,24 @@ make_bundle opts the_s common to_be_sent = make_bundle2 opts the_s common to_be_
 -- patch sequences are passed, a bundle with a mismatched hash will be
 -- generated, which is not the end of the world, but isn't very useful
 -- either.
-make_bundle2 :: RepoPatch p => [DarcsFlag] -> Slurpy -> [PatchInfo]
-             -> FL (Named p) C(x y) -> FL (Named p) C(x y) -> Doc
+make_bundle2 :: RepoPatch p => [DarcsFlag] -> Tree IO -> [PatchInfo]
+             -> FL (Named p) C(x y) -> FL (Named p) C(x y) -> IO Doc
 make_bundle2 opts the_s common to_be_sent to_be_sent2 =
-    text ""
- $$ text "New patches:"
- $$ text ""
- $$ the_new
- $$ text ""
- $$ text "Context:"
- $$ text ""
- $$ (vcat $ map showPatchInfo common)
- $$ text "Patch bundle hash:"
- $$ text (hash_bundle common to_be_sent2)
- $$ text ""
-      where the_new = if Unified `elem` opts
-                      then showContextPatch the_s to_be_sent
-                      else vsep $ mapFL showPatch to_be_sent
+    do patches <- case (isUnified opts) of
+                    True -> fst `fmap` virtualTreeIO (showContextPatch to_be_sent) the_s
+                    False -> return (vsep $ mapFL showPatch to_be_sent)
+       return $ format patches
+    where format the_new = text ""
+                           $$ text "New patches:"
+                           $$ text ""
+                           $$ the_new
+                           $$ text ""
+                           $$ text "Context:"
+                           $$ text ""
+                           $$ (vcat $ map showPatchInfo common)
+                           $$ text "Patch bundle hash:"
+                           $$ text (hash_bundle common to_be_sent2)
+                           $$ text ""
 
 scan_bundle :: RepoPatch p => B.ByteString -> Either String (SealedPatchSet p)
 scan_bundle ps
@@ -84,7 +87,7 @@ scan_bundle ps
         (Sealed patches, rest') ->
             case silly_lex rest' of
             ("Context:", rest'') ->
-                case get_context rest'' of
+                case getContext rest'' of
                 (cont,maybe_hash) ->
                     case substrPS (BC.pack "Patch bundle hash:")
                          maybe_hash of
@@ -102,7 +105,7 @@ scan_bundle ps
             (a,r) -> Left $ "Malformed patch bundle: '"++a++"' is not 'Context:'"
                      ++ "\n" ++ BC.unpack r
     ("Context:",rest) ->
-        case get_context rest of
+        case getContext rest of
         (cont, rest') ->
             case silly_lex rest' of
             ("New patches:", rest'') ->
@@ -153,11 +156,11 @@ pi_unavailable :: RepoPatch p => PatchInfo -> PatchInfoAnd p C(x y)
 pi_unavailable i = (i `patchInfoAndPatch`
                       unavailable ("Patch not stored in patch bundle:\n" ++
                                    renderString (human_friendly i)))
-get_context :: B.ByteString -> ([PatchInfo],B.ByteString)
-get_context ps =
+getContext :: B.ByteString -> ([PatchInfo],B.ByteString)
+getContext ps =
     case readPatchInfo ps of
     Just (pinfo,r') ->
-        case get_context r' of
+        case getContext r' of
         (pis,r'') -> (pinfo:pis, r'')
     Nothing -> ([],ps)
 (-:-) :: a C(x y) -> (Sealed (FL a C(y)),b) -> (Sealed (FL a C(x)),b)
@@ -204,7 +207,7 @@ scan_context ps
   | otherwise =
     case silly_lex ps of
     ("Context:",rest) ->
-        case get_context rest of
+        case getContext rest of
         (cont, _) -> unavailable_patches cont :<: NilRL
     ("-----BEGIN PGP SIGNED MESSAGE-----",rest) ->
             scan_context $ filter_gpg_dashes rest
