@@ -33,10 +33,10 @@ import Data.List ( delete, intersect )
 import Control.Monad ( liftM2 )
 import Control.Monad.Error ( Error(..) )
 
-import Darcs.Patch ( RepoPatch, Named, getdeps, commutex,
+import Darcs.Patch ( RepoPatch, Named, getdeps, commute,
                      commuteFL,
                      patch2patchinfo, merge )
-import Darcs.Ordered ( (:\/:)(..), (:<)(..), (:/\:)(..), (:>)(..),
+import Darcs.Witnesses.Ordered ( (:\/:)(..), (:<)(..), (:/\:)(..), (:>)(..),
                              RL(..), FL(..),
                              (+<+),
                              reverseFL, mapFL_FL, mapFL, concatReverseFL,
@@ -49,15 +49,15 @@ import Darcs.Patch.Patchy ( sloppyIdentity )
 import Darcs.Hopefully ( PatchInfoAnd, piap, info, n2pia,
                          hopefully, conscientiously, hopefullyM )
 import Darcs.ProgressPatches ( progressRL )
-import Darcs.Sealed (Sealed(..), FlippedSeal(..), Sealed2(..)
-                    , flipSeal, seal, unseal )
+import Darcs.Witnesses.Sealed (Sealed(..), FlippedSeal(..), Sealed2(..)
+                    , flipSeal, seal, unseal, mapFlipped )
 import Printer ( errorDoc, renderString, ($$), text )
 #include "impossible.h"
 
 get_common_and_uncommon :: RepoPatch p => (PatchSet p C(x),PatchSet p C(y)) ->
-                           ([PatchInfo],(RL (RL (PatchInfoAnd p)) :\/: RL (RL (PatchInfoAnd p))) C(x y))
+                           ([PatchInfo],(RL (PatchInfoAnd p) :\/: RL (PatchInfoAnd p)) C(x y))
 get_common_and_uncommon_or_missing :: RepoPatch p => (PatchSet p C(x),PatchSet p C(y)) ->
-                                      Either PatchInfo ([PatchInfo],(RL (RL (PatchInfoAnd p)) :\/: RL (RL (PatchInfoAnd p))) C(x y))
+                                      Either PatchInfo ([PatchInfo],(RL (PatchInfoAnd p) :\/: RL (PatchInfoAnd p)) C(x y))
 
 get_common_and_uncommon = 
     either missingPatchError id . get_common_and_uncommon_err
@@ -66,7 +66,7 @@ get_common_and_uncommon_or_missing =
     either (\(MissingPatch x _) -> Left x) Right . get_common_and_uncommon_err
 
 get_common_and_uncommon_err :: RepoPatch p => (PatchSet p C(x),PatchSet p C(y)) ->
-                               Either MissingPatch ([PatchInfo],(RL (RL (PatchInfoAnd p)) :\/: RL (RL (PatchInfoAnd p))) C(x y))
+                               Either MissingPatch ([PatchInfo],(RL (PatchInfoAnd p) :\/: RL (PatchInfoAnd p)) C(x y))
 get_common_and_uncommon_err (ps1,ps2) = gcau (optimize_patchset ps1) ps2
 
 {-|
@@ -184,15 +184,15 @@ that we've got so far.
 -}
 
 gcau :: forall p C(x y). RepoPatch p => PatchSet p C(x) -> PatchSet p C(y)
-     -> Either MissingPatch ([PatchInfo],(RL (RL (PatchInfoAnd p)) :\/: RL (RL (PatchInfoAnd p))) C(x y))
-gcau NilRL ps2 = return ([], NilRL:<:NilRL :\/: concatRL ps2 :<: NilRL)
-gcau ps1 NilRL = return ([], concatRL ps1 :<: NilRL :\/: NilRL:<:NilRL)
+     -> Either MissingPatch ([PatchInfo],(RL (PatchInfoAnd p) :\/: RL (PatchInfoAnd p)) C(x y))
+gcau NilRL ps2 = return ([], NilRL :\/: concatRL ps2)
+gcau ps1 NilRL = return ([], concatRL ps1 :\/: NilRL)
 gcau (NilRL:<:ps1) ps2 = gcau ps1 ps2
 gcau ps1 (NilRL:<:ps2) = gcau ps1 ps2
 gcau ((pi1:<:NilRL):<:_) ((pi2:<:NilRL):<:_)
  | info pi1 == info pi2
  , IsEq <- sloppyIdentity pi1
- , IsEq <- sloppyIdentity pi2 = return ([info pi1], NilRL:<:NilRL :\/: unsafeCoerceP (NilRL:<:NilRL))
+ , IsEq <- sloppyIdentity pi2 = return ([info pi1], NilRL :\/: unsafeCoerceP NilRL)
 gcau (orig_ps1:<:orig_ps1s) (orig_ps2:<:orig_ps2s)
  = f (lengthRL orig_ps1) (unseal info $ lastRL orig_ps1) (orig_ps1:>:NilFL) orig_ps1s
      (lengthRL orig_ps2) (unseal info $ lastRL orig_ps2) (orig_ps2:>:NilFL) orig_ps2s
@@ -200,7 +200,7 @@ gcau (orig_ps1:<:orig_ps1s) (orig_ps2:<:orig_ps2s)
                          lx = last $ concatReverseFL psx   -}
           f :: Int -> PatchInfo -> FL (RL (PatchInfoAnd p)) C(r x) -> PatchSet p C(r)
             -> Int -> PatchInfo -> FL (RL (PatchInfoAnd p)) C(u y) -> PatchSet p C(u)
-            -> Either MissingPatch ([PatchInfo],(RL (RL (PatchInfoAnd p)) :\/: RL (RL (PatchInfoAnd p))) C(x y))
+            -> Either MissingPatch ([PatchInfo],(RL (PatchInfoAnd p) :\/: RL (PatchInfoAnd p)) C(x y))
           f _n1 l1 ps1 _ps1s _n2 l2 ps2 _ps2s
            | l1 == l2 = gcau_simple (unsafeCoerceP (concatReverseFL ps1)) (unsafeCoerceP (concatReverseFL ps2))
           f n1 l1 ps1 ps1s n2 l2 ps2 ps2s
@@ -242,12 +242,12 @@ dropWhileNilRL xs = xs
 -- | Filters the common elements from @ps1@ and @ps2@ and returns the simplified sequences.
 gcau_simple :: RepoPatch p => RL (PatchInfoAnd p) C(x y) -- ^ @ps1@
             -> RL (PatchInfoAnd p) C(u v) -- ^ @ps2@
-            -> Either MissingPatch ([PatchInfo],(RL (RL (PatchInfoAnd p)) :\/: RL (RL (PatchInfoAnd p))) C(y v))
+            -> Either MissingPatch ([PatchInfo],(RL (PatchInfoAnd p) :\/: RL (PatchInfoAnd p)) C(y v))
 gcau_simple ps1 ps2 = do
  FlippedSeal ex1 <- get_extra common ps1
  FlippedSeal ex2 <- get_extra common ps2
  let ps1' = filter (`elem` common) $ ps1_info
- return (ps1', (unsafeCoerceP ex1 :<: NilRL) :\/: ex2 :<: NilRL)
+ return (ps1', (unsafeCoerceP ex1 :\/: ex2))
   where common   = ps1_info `intersect` mapRL info ps2
         ps1_info = mapRL info ps1
 
@@ -258,7 +258,33 @@ instance Error MissingPatch where
     noMsg = bug "MissingPatch doesn't define noMsg."
 
 -- | Returns a sub-sequence from @patches@, where all the elements of @common@ have
--- been removed by commuting them out.
+--   been removed by commuting them back into the early part of the history.
+--
+--  An informal illustration of this process as it traverses a mixed list of patches
+--  where C and x denote common patches and extra patches accordingly.  Variants of
+--  patches obtained through commutation are indicated by letters following the
+--  patch name.
+--
+--  > in: x6 < x5 < C4 < x3 < C2 < x1  skip:           extra:
+--  > in:      x5 < C4 < x3 < C2 < x1  skip:           extra:                  x6
+--  > in:           C4 < x3 < C2 < x1  skip:           extra:             x5 > x6
+--  > in:                x3 < C2 < x1  skip: C4        extra:             x5 > x6
+--  > in:                     C2 < x1  skip: C4b       extra:       x3b > x5 > x6
+--  > in:                          x1  skip: C2  > C4b extra:       x3b > x5 > x6
+--  > in:                              skip: C2b > C4c extra: x1b > x3b > x5 > x6
+--
+--  This function is undefined if for any reason we fail to commute an extra
+--  patch past one of the common ones.  Such a failure would indicate that the
+--  common patch depends on the extra one, contradicting the claim that the
+--  \"common\" patch is shared with another repository lacking the extra patches.
+--  Unfortunately, such cases have crept up in practice.  Some notable cases can
+--  be found on the bugtracker as:
+--
+--   * issue27   - different patches with identical patch info; mistaken identity.
+--                 Note how @common@ consists only of a list of 'PatchInfo' which
+--                 we trust to uniquely identify such patches.
+--
+--   * issue1014 - duplicate patches
 get_extra :: RepoPatch p => [PatchInfo] -- ^ @common@
           -> RL (PatchInfoAnd p) C(u x) -- ^ @patches@
           -> Either MissingPatch (FlippedSeal (RL (PatchInfoAnd p)) C(y))
@@ -313,19 +339,14 @@ get_extra_old :: RepoPatch p => [PatchInfo]
 get_extra_old common pps =
     either missingPatchError id (get_extra common pps)
 
-get_patches_beyond_tag :: RepoPatch p => PatchInfo -> PatchSet p C(x) -> FlippedSeal (RL (RL (PatchInfoAnd p))) C(x)
-get_patches_beyond_tag t ((hp:<:NilRL):<:_) | info hp == t = flipSeal $ NilRL :<: NilRL
+get_patches_beyond_tag :: RepoPatch p => PatchInfo -> PatchSet p C(x) -> FlippedSeal (RL (PatchInfoAnd p)) C(x)
+get_patches_beyond_tag t ((hp:<:NilRL):<:_) | info hp == t = flipSeal NilRL
 get_patches_beyond_tag t patchset@((hp:<:ps):<:pps) =
     if info hp == t
     then if get_tags_right patchset == [info hp]
-         then flipSeal $ NilRL :<: NilRL -- special case to avoid looking at redundant patches
-         else case get_extra_old [t] (concatRL patchset) of
-              FlippedSeal x -> flipSeal $ x :<: NilRL
-    else hp `prepend` get_patches_beyond_tag t (ps:<:pps)
- where
- prepend :: (PatchInfoAnd p) C(x y) -> FlippedSeal (RL (RL (PatchInfoAnd p))) C(x) -> FlippedSeal (RL (RL (PatchInfoAnd p))) C(y)
- prepend pp (FlippedSeal NilRL)     = flipSeal $ (pp:<:NilRL) :<: NilRL
- prepend pp (FlippedSeal (p:<:ps')) = flipSeal $ (pp:<:p)     :<: ps'
+         then flipSeal NilRL -- special case to avoid looking at redundant patches
+         else get_extra_old [t] (concatRL patchset)
+    else mapFlipped (hp:<:) $ get_patches_beyond_tag t (ps:<:pps)
 get_patches_beyond_tag t (NilRL:<:pps) = get_patches_beyond_tag t pps
 get_patches_beyond_tag t NilRL = bug $ "tag\n" ++
                                  renderString (human_friendly t) ++
@@ -367,9 +388,9 @@ get_patches_in_tag t ((pa:<:ps):<:xs) = gpit thepis (pa:>:NilFL) (ps:<:xs)
                      -> FL (PatchInfoAnd p) C(w z)
           commute_by NilFL _ = unsafeCoerceP NilFL
           commute_by (hpa:>:xs') p =
-              case commutex (hopefully hpa :< p) of
+              case commute (p :> hopefully hpa) of
                 Nothing -> bug "Failure commuting patches in commute_by called by gpit!"
-                Just (p':<a') -> (info hpa `piap` a') :>: commute_by xs' p'
+                Just (a' :> p') -> (info hpa `piap` a') :>: commute_by xs' p'
 
 get_patches_in_tag t _ = errorDoc $ text "Couldn't read tag"
                                  $$ human_friendly t
@@ -481,13 +502,12 @@ patchset_union (Sealed y:ys) =
   f common a b = g_s $ gcau_simple a b
     where
       g_s :: Either MissingPatch
-                    ([PatchInfo],(RL (RL (PatchInfoAnd p)) :\/: RL (RL (PatchInfoAnd p))) C(x y))
+                    ([PatchInfo],(RL (PatchInfoAnd p) :\/: RL (PatchInfoAnd p)) C(x y))
           -> SealedPatchSet p
       g_s (Left e) = missingPatchError e
-      g_s (Right (_, (a' :<: NilRL) :\/: (b' :<: NilRL))) =
+      g_s (Right (_, a' :\/: b')) =
           case (merge_sets (a' :\/: b')) of
           Sealed a'b' -> seal $ (a'b' +<+ b) :<: common
-      g_s _ = impossible
 
 merge_sets :: RepoPatch p => (RL (PatchInfoAnd p) :\/: RL (PatchInfoAnd p)) C(x y) -> Sealed (RL (PatchInfoAnd p) C(y))
 merge_sets (l :\/: r) =
