@@ -21,7 +21,7 @@ module Darcs.Patch.FileName ( FileName( ),
                               fp2fn, fn2fp,
                               fn2ps, ps2fn,
                               niceps2fn, fn2niceps,
-                              break_on_dir, norm_path, own_name, super_name,
+                              break_on_dir, norm_path, own_name, superName,
                               movedirfilename,
                               encode_white, decode_white,
                               (///),
@@ -30,9 +30,9 @@ module Darcs.Patch.FileName ( FileName( ),
 
 import System.IO
 import Data.Char ( isSpace, chr, ord )
-import qualified UTF8 ( encode )
+import qualified Codec.Binary.UTF8.String as UTF8 ( encode )
+import Codec.Binary.UTF8.Generic ( toString )
 import Data.Word ( Word8( ) )
-import ByteStringUtils ( unpackPSfromUTF8 )
 import qualified Data.ByteString.Char8 as BC (unpack, pack)
 import qualified Data.ByteString       as B  (ByteString, pack)
 
@@ -66,7 +66,7 @@ fn2ps (FN fp) = B.pack $ encode $ encode_white fp
 
 {-# INLINE ps2fn #-}
 ps2fn :: B.ByteString -> FileName
-ps2fn ps = FN $ decode_white $ unpackPSfromUTF8 ps
+ps2fn ps = FN $ decode_white $ toString ps
 
 -- | 'encode_white' translates whitespace in filenames to a darcs-specific
 --   format (backslash followed by numerical representation according to 'ord').
@@ -98,8 +98,8 @@ decode_white "" = ""
 own_name :: FileName -> FileName
 own_name (FN f) = case breakLast '/' f of Nothing -> FN f
                                           Just (_,f') -> FN f'
-super_name :: FileName -> FileName
-super_name fn = case norm_path fn of
+superName :: FileName -> FileName
+superName fn = case norm_path fn of
                 FN f -> case breakLast '/' f of
                         Nothing -> FN "."
                         Just (d,_) -> FN d
