@@ -1,4 +1,5 @@
 -- Copyright (C) 2002-2003 David Roundy
+-- Copyright (C) 2009 Ganesh Sittampalam
 --
 -- This program is free software; you can redistribute it and/or modify
 -- it under the terms of the GNU General Public License as published by
@@ -27,11 +28,14 @@ module Darcs.Patch.Permutations ( removeFL, removeRL, removeCommon,
                                   partitionFL, partitionRL,
                                   head_permutationsFL, head_permutationsRL,
                                   headPermutationsFL,
-                                  remove_subsequenceFL, remove_subsequenceRL ) where
+                                  remove_subsequenceFL, remove_subsequenceRL,
+                                  partitionConflictingFL,
+                                  CommuteFn, selfCommuter, commuterIdRL,
+                                ) where
 
 import Data.Maybe ( catMaybes )
 import Darcs.Patch.Patchy ( Commute, commute, commuteFL, commuteRL, Invert(..), invertFL, invertRL )
-import Darcs.Ordered
+import Darcs.Witnesses.Ordered
 #include "impossible.h"
 
 -- |split an 'FL' into "left" and "right" lists according to a predicate, using commutation as necessary.
@@ -230,3 +234,37 @@ instance (Commute p, Invert p) => Invert (RL p) where
     sloppyIdentity NilRL = IsEq
     sloppyIdentity (x:<:xs) | IsEq <- sloppyIdentity x = sloppyIdentity xs
     sloppyIdentity _ = NotEq
+
+-- |CommuteFn is the basis of a general framework for building up commutation
+-- operations between different patch types in a generic manner. Unfortunately
+-- type classes are not well suited to the problem because of the multiple possible
+-- routes by which the commuter for (FL p1, FL p2) can be built out of the
+-- commuter for (p1, p2) - and more complicated problems when we start building
+-- multiple constructors on top of each other. The type class resolution machinery
+-- really can't cope with selecting some route, because it doesn't know that all
+-- possible routes should be equivalent.
+type CommuteFn p1 p2 = FORALL(x y) (p1 :> p2) C(x y) -> Maybe ((p2 :> p1) C(x y))
+
+-- |Build a commuter between a patch and itself using the operation from the type class.
+selfCommuter :: Commute p => CommuteFn p p
+selfCommuter = commute
+
+commuterIdRL :: CommuteFn p1 p2 -> CommuteFn p1 (RL p2)
+commuterIdRL _ (x :> NilRL) = return (NilRL :> x)
+commuterIdRL commuter (x :> (y :<: ys))
+  = do ys' :> x' <- commuterIdRL commuter (x :> ys)
+       y' :> x'' <- commuter (x' :> y)
+       return ((y' :<: ys') :> x'')
+
+-- |Partition a list into the patches that commute with the given patch and those that don't (including dependencies)
+partitionConflictingFL :: (Commute p1, Invert p1) => CommuteFn p1 p2 -> FL p1 C(x y) -> p2 C(x z) -> (FL p1 :> FL p1) C(x y)
+partitionConflictingFL _ NilFL _ = (NilFL :> NilFL)
+partitionConflictingFL commuter (x :>: xs) y =
+   case commuter (invert x :> y) of
+     Nothing -> case commuteWhatWeCanFL (x :> xs) of
+                 xs_ok :> x' :> xs_deps ->
+                   case partitionConflictingFL commuter xs_ok y of
+                     xs_clean :> xs_conflicts -> xs_clean :> (xs_conflicts +>+ (x' :>: xs_deps))
+     Just (y' :> _) ->
+       case partitionConflictingFL commuter xs y' of
+          xs_clean :> xs_conflicts -> (x :>: xs_clean) :> xs_conflicts
