@@ -33,7 +33,7 @@ import Darcs.Patch.Prim ( Prim, FromPrim(..), ToFromPrim(..), Conflict(..), Effe
                           IsConflictedPrim(..), ConflictState(..) )
 import Darcs.Patch.Read ( readPrim )
 import Darcs.Patch.Patchy
-import Darcs.Ordered
+import Darcs.Witnesses.Ordered
 --import Darcs.Patch.Read ()
 --import Darcs.Patch.Viewing ()
 --import Darcs.Patch.Apply ()
@@ -45,11 +45,11 @@ import Darcs.Patch.Non ( Non(..), Nonable(..), unNon,
 import Darcs.Patch.Permutations ( commuteWhatWeCanFL, commuteWhatWeCanRL,
                                   genCommuteWhatWeCanRL,
                                   removeRL, removeFL, remove_subsequenceFL )
-import qualified Data.ByteString.Char8 as BC ( unpack )
-import Darcs.Patch.ReadMonads (work, peek_input, my_lex )
+import qualified Data.ByteString.Char8 as BC ( ByteString, unpack )
+import Darcs.Patch.ReadMonads ( work, peek_input, my_lex )
 import Darcs.Utils ( nubsort )
-import Darcs.Sealed ( FlippedSeal(..), Sealed(Sealed), mapSeal )
-import Darcs.Show
+import Darcs.Witnesses.Sealed ( FlippedSeal(..), Sealed(Sealed), mapSeal )
+import Darcs.Witnesses.Show
 import Printer ( Doc, renderString, blueText, redText, (<+>), ($$) )
 import Darcs.ColorPrinter ( errorDoc, assertDoc )
 --import Printer ( greenText )
@@ -291,7 +291,7 @@ instance Conflict RealPatch where
     conflictedEffect (Conflictor _ _ (Non _ x)) = [IsC Conflicted x]
     conflictedEffect (InvConflictor _ _ _) = impossible
     conflictedEffect (Normal x) = [IsC Okay x]
-    resolve_conflicts (Conflictor ix xx x) = [mangle_unravelled unravelled : unravelled]
+    resolveConflicts (Conflictor ix xx x) = [mangle_unravelled unravelled : unravelled]
             where unravelled = nub $ filter isn $ map (`merge_with` (x:ix++nonxx)) (x:ix++nonxx)
                   nonxx = nonxx_ (nonxx_aux ix xx)
                   nonxx_aux :: [Non RealPatch C(x)] -> FL Prim C(x y) -> RL RealPatch C(x y)
@@ -303,7 +303,7 @@ instance Conflict RealPatch where
                   isn :: Sealed (FL p C(x)) -> Bool
                   isn (Sealed NilFL) = False
                   isn _ = True
-    resolve_conflicts _ = []
+    resolveConflicts _ = []
 
     -- cA
     commute_no_conflicts (Duplicate x :> Duplicate y) = Just (Duplicate y :> Duplicate x)
@@ -406,7 +406,10 @@ invertNon (Non c x)
     where nix = Normal $ invert x
 
 nonTouches :: Non RealPatch C(x) -> [FilePath]
-nonTouches (Non c x) = list_touched_files (c +>+ fromPrim x :>: NilFL)
+nonTouches (Non c x) = listTouchedFiles (c +>+ fromPrim x :>: NilFL)
+
+nonHunkMatches :: (BC.ByteString -> Bool) -> Non RealPatch C(x) -> Bool
+nonHunkMatches f (Non c x) = hunkMatches f c || hunkMatches f x
 
 toNons :: (Conflict p, Patchy p, ToFromPrim p, Nonable p) => FL p C(x y) -> [Non p C(x)]
 toNons xs = map lastNon $ initsFL xs
@@ -592,13 +595,19 @@ instance Commute RealPatch where
             Nothing -> impossible pullInContext fromNons
 --    merge _ = error "haven't finished fixing merge"
 
-    list_touched_files (Duplicate p) = nonTouches p
-    list_touched_files (Etacilpud p) = nonTouches p
-    list_touched_files (Normal p) = list_touched_files p
-    list_touched_files (Conflictor x c p) =
-        nubsort $ concatMap nonTouches x ++ list_touched_files c ++ nonTouches p
-    list_touched_files (InvConflictor x c p) =
-        nubsort $ concatMap nonTouches x ++ list_touched_files c ++ nonTouches p
+    listTouchedFiles (Duplicate p) = nonTouches p
+    listTouchedFiles (Etacilpud p) = nonTouches p
+    listTouchedFiles (Normal p) = listTouchedFiles p
+    listTouchedFiles (Conflictor x c p) =
+        nubsort $ concatMap nonTouches x ++ listTouchedFiles c ++ nonTouches p
+    listTouchedFiles (InvConflictor x c p) =
+        nubsort $ concatMap nonTouches x ++ listTouchedFiles c ++ nonTouches p
+
+    hunkMatches f (Duplicate p) = nonHunkMatches f p
+    hunkMatches f (Etacilpud p) = nonHunkMatches f p
+    hunkMatches f (Normal p) = hunkMatches f p
+    hunkMatches f (Conflictor x c p) = or [or $ map (nonHunkMatches f) x, hunkMatches f c, nonHunkMatches f p]
+    hunkMatches f (InvConflictor x c p) = or [or $ map (nonHunkMatches f) x, hunkMatches f c, nonHunkMatches f p]
 
 {-
 all_conflicts_withFL :: FL Prim C(x y) -> [Non RealPatch C(x)]
@@ -714,11 +723,11 @@ instance ShowPatch RealPatch where
         showPatch cs $$
         blueText "]" $$
         showNon p
-    showContextPatch s (Normal p) = showContextPatch s p
-    showContextPatch _ c = showPatch c
+    showContextPatch (Normal p) = showContextPatch p
+    showContextPatch c = return $ showPatch c
 
 instance ReadPatch RealPatch where
- readPatch' want_eof =
+ readPatch' _ =
      do s <- peek_input
         case fmap (BC.unpack . fst) $ my_lex s of
           Just "duplicate" ->
@@ -744,7 +753,7 @@ instance ReadPatch RealPatch where
                  Just (Sealed ps) <- bracketedFL (fromIntegral $ fromEnum '[') (fromIntegral $ fromEnum ']')
                  Just p <- readNon
                  return $ Just $ Sealed $ InvConflictor i ps p
-          _ -> do mp <- readPrim NewFormat want_eof
+          _ -> do mp <- readPrim NewFormat
                   case mp of
                     Just p -> return $ Just $ Normal `mapSeal` p
                     Nothing -> return Nothing
