@@ -23,8 +23,8 @@
 module Darcs.Patch.Set ( PatchSet, SealedPatchSet ) where
 
 import Darcs.Hopefully ( PatchInfoAnd )
-import Darcs.Ordered ( RL )
-import Darcs.Sealed ( Sealed )
+import Darcs.Witnesses.Ordered ( RL )
+import Darcs.Witnesses.Sealed ( Sealed )
 
 -- | A PatchSet is in reverse order, plus has information about which
 -- tags are clean, meaning all patches applied prior to them are in
