@@ -21,56 +21,47 @@
 
 #include "gadts.h"
 
-module Darcs.Repository ( Repository, ($-), maybeIdentifyRepository,
-                          identifyRepositoryFor,
-                          withRepoLock, withRepoReadLock,
-                          withRepository, withRepositoryDirectory, withGutsOf,
-                          makePatchLazy, writePatchSet,
-                    findRepository, amInRepository, amNotInRepository,
-                    slurp_pending, replacePristineFromSlurpy,
-                    slurp_recorded, slurp_recorded_and_unrecorded,
-                    withRecorded,
-                    get_unrecorded, get_unrecorded_unsorted, get_unrecorded_no_look_for_adds,
-                    get_unrecorded_in_files, get_unrecorded_in_files_unsorted,
-                    read_repo, sync_repo,
-                    prefsUrl,
-                    add_to_pending,
-                    tentativelyAddPatch, tentativelyRemovePatches, tentativelyAddToPending,
-                    tentativelyReplacePatches,
-                    tentativelyMergePatches, considerMergeToWorking,
-                    revertRepositoryChanges, finalizeRepositoryChanges,
-                          createRepository, copyRepository, copy_oldrepo_patches,
-                    patchSetToRepository,
-                    unrevertUrl,
-                    applyToWorking, patchSetToPatches,
-                    createPristineDirectoryTree, createPartialsPristineDirectoryTree,
-                    optimizeInventory, cleanRepository,
-                    checkPristineAgainstSlurpy, getMarkedupFile,
-                    PatchSet, SealedPatchSet, PatchInfoAnd,
-                    setScriptsExecutable,
-                    checkUnrelatedRepos,
-                    testTentative, testRecorded
-                  ) where
+module Darcs.Repository
+    ( Repository, ($-), maybeIdentifyRepository, identifyRepositoryFor
+    , withRepoLock, withRepoReadLock, withRepository, withRepositoryDirectory
+    , withGutsOf, makePatchLazy, writePatchSet, findRepository, amInRepository
+    , amNotInRepository, slurp_pending, replacePristine, slurp_recorded
+    , slurp_recorded_and_unrecorded, withRecorded, read_repo, prefsUrl
+    , add_to_pending, tentativelyAddPatch, tentativelyRemovePatches
+    , tentativelyAddToPending, tentativelyReplacePatches
+    , tentativelyMergePatches, considerMergeToWorking, revertRepositoryChanges
+    , finalizeRepositoryChanges, createRepository, copyRepository
+    , copy_oldrepo_patches, patchSetToRepository, unrevertUrl, applyToWorking
+    , patchSetToPatches, createPristineDirectoryTree
+    , createPartialsPristineDirectoryTree, optimizeInventory, cleanRepository
+    , getMarkedupFile, PatchSet, SealedPatchSet, PatchInfoAnd
+    , setScriptsExecutable, checkUnrelatedRepos, testTentative, testRecorded
+    -- * Recorded and unrecorded and pending.
+    , readRecorded, readUnrecorded, unrecordedChanges, readPending, pendingChanges
+    , readRecordedAndPending
+    -- * Index.
+    , readIndex, invalidateIndex
+    ) where
 
 import System.Exit ( ExitCode(..), exitWith )
 
+import Darcs.Repository.State( readRecorded, readUnrecorded, readWorking, unrecordedChanges
+                             , readPending, pendingChanges, readIndex, invalidateIndex
+                             , readRecordedAndPending )
+
 import Darcs.Repository.Internal
-    (Repository(..), RepoType(..), ($-), pristineFromWorking,
+    (Repository(..), RepoType(..), ($-),
      maybeIdentifyRepository, identifyRepositoryFor,
      findRepository, amInRepository, amNotInRepository,
      makePatchLazy,
-     slurp_pending, replacePristineFromSlurpy,
+     slurp_pending,
      slurp_recorded, slurp_recorded_and_unrecorded,
      withRecorded,
-     get_unrecorded, get_unrecorded_unsorted, get_unrecorded_no_look_for_adds,
-     get_unrecorded_in_files, get_unrecorded_in_files_unsorted,
-     read_repo, sync_repo,
-     prefsUrl, checkPristineAgainstSlurpy,
-     add_to_pending,
+     read_repo,
+     prefsUrl,
      withRepoLock, withRepoReadLock, withRepository, withRepositoryDirectory, withGutsOf,
      tentativelyAddPatch, tentativelyRemovePatches, tentativelyAddToPending,
      tentativelyReplacePatches,
-     tentativelyMergePatches, considerMergeToWorking,
      revertRepositoryChanges, finalizeRepositoryChanges,
      unrevertUrl,
      applyToWorking, patchSetToPatches,
@@ -78,14 +69,16 @@ import Darcs.Repository.Internal
      optimizeInventory, cleanRepository,
      getMarkedupFile,
      setScriptsExecutable,
-     testTentative, testRecorded
+     testTentative, testRecorded,
+     make_new_pending
     )
+import Darcs.Repository.Merge( tentativelyMergePatches, considerMergeToWorking )
 import Darcs.Repository.Cache ( unionCaches, fetchFileUsingCache, HashedDir(..) )
 import Darcs.Patch.Set ( PatchSet, SealedPatchSet )
 
 import Control.Monad ( unless, when )
 import Data.Either(Either(..))
-import System.Directory ( createDirectory )
+import System.Directory ( createDirectory, renameDirectory )
 import System.IO.Error ( isAlreadyExistsError )
 
 import qualified Darcs.Repository.DarcsRepo as DarcsRepo
@@ -94,30 +87,40 @@ import qualified Darcs.Repository.HashedRepo as HashedRepo
 import Darcs.Hopefully ( PatchInfoAnd, info, extractHash )
 import Darcs.Repository.Checkpoint ( identify_checkpoint, write_checkpoint_patch, get_checkpoint )
 import Darcs.Repository.ApplyPatches ( apply_patches )
-import Darcs.Repository.HashedRepo ( apply_to_tentative_pristine )
-import Darcs.Patch ( RepoPatch, Named, Patch, patch2patchinfo, apply )
-import Darcs.Ordered ( RL(..), bunchFL, mapFL, mapRL, mapRL_RL, concatFL, reverseRL,
-                       concatRL, lengthRL, isShorterThanRL )
+import Darcs.Repository.HashedRepo ( apply_to_tentative_pristine, pris2inv )
+import Darcs.Repository.InternalTypes ( Pristine(..) )
+import Darcs.Patch ( RepoPatch, Named, Prim, Patch, patch2patchinfo, apply )
+import Darcs.Witnesses.Ordered ( FL(..), RL(..), bunchFL, mapFL, mapRL, mapRL_RL, concatFL
+                     , reverseRL ,concatRL, lengthRL, isShorterThanRL, (+>+) )
 import Darcs.Patch.Info ( PatchInfo )
 import Darcs.Repository.Format ( RepoProperty ( HashedInventory ),
-                                 create_repo_format, format_has, writeRepoFormat )
-import Darcs.Repository.Prefs ( write_default_prefs )
-import Darcs.Repository.Pristine ( createPristine, flagsToPristine )
+                                 createRepoFormat, formatHas, writeRepoFormat )
+import Darcs.Repository.Prefs ( writeDefaultPrefs )
+import Darcs.Repository.Pristine ( createPristine, flagsToPristine, createPristineFromWorking )
 import Darcs.Patch.Depends ( get_patches_beyond_tag )
-import Darcs.SlurpDirectory ( empty_slurpy )
 import Darcs.Utils ( withCurrentDirectory, catchall, promptYorn, prettyError )
 import Darcs.External ( copyFileOrUrl, Cachable(..) )
 import Progress ( debugMessage, tediousSize,
                         beginTedious, endTedious, progress )
 import Darcs.ProgressPatches (progressRLShowTags, progressFL)
-import Darcs.Lock ( writeBinFile )
-import Darcs.Sealed ( Sealed(..), FlippedSeal(..), flipSeal, mapFlipped )
+import Darcs.Lock ( writeBinFile, writeDocBinFile, rm_recursive )
+import Darcs.Witnesses.Sealed ( Sealed(..), FlippedSeal(..), flipSeal, mapFlipped )
 
-import Darcs.Flags ( DarcsFlag( Quiet, Partial, Lazy, Ephemeral,
-                                AllowUnrelatedRepos
-                              ),
-                     compression )
+import Darcs.Flags ( DarcsFlag( Quiet, Partial, Lazy, Ephemeral, Complete,
+                                AllowUnrelatedRepos, NoUpdateWorking )
+                   , compression )
 import Darcs.Global ( darcsdir )
+
+import Storage.Hashed.Tree( Tree, emptyTree )
+import Storage.Hashed.Hash( encodeBase16 )
+import Storage.Hashed.Darcs( writeDarcsHashed, darcsAddMissingHashes )
+import Storage.Hashed( writePlainTree )
+import ByteStringUtils( gzReadFilePS )
+
+import System.FilePath( (</>) )
+
+import qualified Data.ByteString.Char8 as BS
+
 #include "impossible.h"
 
 createRepository :: [DarcsFlag] -> IO ()
@@ -126,19 +129,19 @@ createRepository opts = do
       (\e-> if isAlreadyExistsError e
             then fail "Tree has already been initialized!"
             else fail $ "Error creating directory `"++darcsdir++"'.")
-  let rf = create_repo_format opts
+  let rf = createRepoFormat opts
   createPristine $ flagsToPristine opts rf
   createDirectory $ darcsdir ++ "/patches"
   createDirectory $ darcsdir ++ "/prefs"
-  write_default_prefs
+  writeDefaultPrefs
   writeRepoFormat rf (darcsdir++"/format")
-  if format_has HashedInventory rf
+  if formatHas HashedInventory rf
       then writeBinFile (darcsdir++"/hashed_inventory") ""
       else DarcsRepo.write_inventory "." ((NilRL:<:NilRL) :: PatchSet Patch C(())) -- YUCK!
 
 copyRepository :: RepoPatch p => Repository p C(r u t) -> IO ()
 copyRepository fromrepository@(Repo _ opts rf _)
-    | Partial `elem` opts && not (format_has HashedInventory rf) =
+    | Partial `elem` opts && not (formatHas HashedInventory rf) =
         do isPartial <- copyPartialRepository fromrepository
            unless (isPartial == IsPartial) $ copyFullRepository fromrepository
     | otherwise = copyFullRepository fromrepository
@@ -156,11 +159,11 @@ copyInventory fromrepo@(Repo fromdir opts rf (DarcsRepository _ cremote)) = do
       copyHashedHashed = HashedRepo.copy_repo newrepo opts fromdir
       copyAnythingToOld r = withCurrentDirectory todir $ read_repo r >>=
                             DarcsRepo.write_inventory_and_patches opts
-      repoSort rfx | format_has HashedInventory rfx = Hashed
+      repoSort rfx | formatHas HashedInventory rfx = Hashed
                    | otherwise = Old
   case repoSort rf2 of
     Hashed ->
-        if format_has HashedInventory rf
+        if formatHas HashedInventory rf
         then copyHashedHashed
         else withCurrentDirectory todir $
              do HashedRepo.revert_tentative_changes
@@ -207,7 +210,7 @@ copyPartialRepository fromrepository@(Repo _ opts _ _) = do
            local_patches <- read_repo torepository
            let pi_ch = patch2patchinfo ch
            FlippedSeal ps <- return $ get_patches_beyond_tag pi_ch local_patches
-           let needed_patches = reverseRL $ concatRL ps
+           let needed_patches = reverseRL ps
            apply opts ch `catch`
                              \e -> fail ("Bad checkpoint!\n" ++ prettyError e)
            apply_patches opts needed_patches
@@ -223,15 +226,15 @@ copyFullRepository fromrepository@(Repo fromdir opts rffrom _) = do
                      `catchall` return ()
   debugMessage "Grabbing lock in new repository..."
   withRepoLock opts $- \torepository@(Repo _ _ rfto (DarcsRepository _ c)) ->
-      if format_has HashedInventory rffrom && format_has HashedInventory rfto
+      if formatHas HashedInventory rffrom && formatHas HashedInventory rfto
       then do debugMessage "Writing working directory contents..."
               createPristineDirectoryTree torepository "."
               fetch_patches_if_necessary opts torepository
               when (Partial `elem` opts) $ putStrLn $
                        "--partial: hashed or darcs-2 repository detected, using --lazy instead"
-      else if format_has HashedInventory rfto
+      else if formatHas HashedInventory rfto
            then do local_patches <- read_repo torepository
-                   replacePristineFromSlurpy torepository empty_slurpy
+                   replacePristine torepository emptyTree
                    let patchesToApply = progressFL "Applying patch" $ concatFL $ reverseRL $
                                         mapRL_RL reverseRL local_patches
                    sequence_ $ mapFL (apply_to_tentative_pristine c opts) $ bunchFL 100 patchesToApply
@@ -252,7 +255,7 @@ writePatchSet patchset opts = do
             Right r -> r
             Left e  -> bug ("Current directory not repository in writePatchSet: " ++ e)
     debugMessage "Writing inventory"
-    if format_has HashedInventory rf2
+    if formatHas HashedInventory rf2
        then do HashedRepo.write_tentative_inventory c (compression opts) patchset
                HashedRepo.finalize_tentative_changes repo (compression opts)
        else DarcsRepo.write_inventory_and_patches opts patchset
@@ -266,7 +269,7 @@ writePatchSet patchset opts = do
 patchSetToRepository :: RepoPatch p => Repository p C(r1 u1 r1) -> PatchSet p C(x)
                      -> [DarcsFlag] -> IO (Repository p C(r u t))
 patchSetToRepository (Repo fromrepo _ rf _) patchset opts = do
-    when (format_has HashedInventory rf) $ -- set up sources and all that
+    when (formatHas HashedInventory rf) $ -- set up sources and all that
        do writeFile "_darcs/tentative_pristine" "" -- this is hokey
           repox <- writePatchSet patchset opts
           HashedRepo.copy_repo repox opts fromrepo
@@ -276,7 +279,7 @@ patchSetToRepository (Repo fromrepo _ rf _) patchset opts = do
     pristineFromWorking repo
     return repo
 
-checkUnrelatedRepos :: [DarcsFlag] -> [PatchInfo] -> PatchSet p C(x) -> PatchSet p C(x) -> IO ()
+checkUnrelatedRepos :: [DarcsFlag] -> [PatchInfo] -> PatchSet p C(x) -> PatchSet p C(y) -> IO ()
 checkUnrelatedRepos opts common us them
     | AllowUnrelatedRepos `elem` opts || not (null common)
        || concatRL us `isShorterThanRL` 5 || concatRL them `isShorterThanRL` 5
@@ -292,7 +295,8 @@ checkUnrelatedRepos opts common us them
 fetch_patches_if_necessary :: RepoPatch p => [DarcsFlag] -> Repository p C(r u t) -> IO ()
 fetch_patches_if_necessary opts torepository@(Repo _ _ _ (DarcsRepository _ c)) = 
     unless (Partial `elem` opts || Lazy `elem` opts || Ephemeral `elem` opts) $
-             do putInfo "Copying patches, to get lazy repository hit ctrl-C..."
+             do unless (Complete `elem` opts) $
+                       putInfo "Copying patches, to get lazy repository hit ctrl-C..."
                 r <- read_repo torepository
                 let peekaboo :: PatchInfoAnd p C(x y) -> IO ()
                     peekaboo x = case extractHash x of
@@ -300,3 +304,38 @@ fetch_patches_if_necessary opts torepository@(Repo _ _ _ (DarcsRepository _ c))
                                  Right h -> fetchFileUsingCache c HashedPatchesDir h >> return ()
                 sequence_ $ mapRL peekaboo $ progressRLShowTags "Copying patches" $ concatRL r
   where putInfo = when (not $ Quiet `elem` opts) . putStrLn
+
+add_to_pending :: RepoPatch p => Repository p C(r u t) -> FL Prim C(u y) -> IO ()
+add_to_pending (Repo _ opts _ _) _ | NoUpdateWorking `elem` opts = return ()
+add_to_pending repo@(Repo _ opts _ _) p =
+    do pend <- unrecordedChanges opts repo []
+       invalidateIndex repo
+       make_new_pending repo (pend +>+ p)
+
+-- | Replace the existing pristine with a new one (loaded up in a Tree object).
+replacePristine :: Repository p C(r u t) -> Tree IO -> IO ()
+replacePristine (Repo r _opts _rf (DarcsRepository pris _c)) tree =
+    withCurrentDirectory r $ replace pris
+    where replace HashedPristine =
+              do let t = darcsdir </> "hashed_inventory"
+                 i <- gzReadFilePS t
+                 tree' <- darcsAddMissingHashes tree
+                 root <- writeDarcsHashed tree' $ darcsdir </> "pristine.hashed"
+                 writeDocBinFile t $ pris2inv (BS.unpack $ encodeBase16 root) i
+          replace (PlainPristine n) =
+              do rm_recursive nold `catchall` return ()
+                 writePlainTree tree ntmp
+                 renameDirectory n nold
+                 renameDirectory ntmp n
+                 return ()
+          replace (NoPristine _) = return ()
+          nold = darcsdir </> "pristine-old"
+          ntmp = darcsdir </> "pristine-tmp"
+
+pristineFromWorking :: RepoPatch p => Repository p C(r u t) -> IO ()
+pristineFromWorking repo@(Repo dir _ rf _)
+    | formatHas HashedInventory rf =
+        withCurrentDirectory dir $ readWorking >>= replacePristine repo
+pristineFromWorking (Repo dir _ _ (DarcsRepository p _)) =
+  withCurrentDirectory dir $ createPristineFromWorking p
+
