@@ -9,7 +9,7 @@ module Darcs.Repository.Cache (
                    HashedDir(..), hashedDir,
                    unionCaches, cleanCaches, cleanCachesWithHint,
                    fetchFileUsingCache, speculateFileUsingCache, writeFileUsingCache,
-                   findFileMtimeUsingCache, setFileMtimeUsingCache, peekInCache,
+                   peekInCache,
                    repo2cache,
                    writable, isthisrepo, hashedFilePath, allHashedDirs
                  ) where
@@ -18,9 +18,7 @@ import Control.Monad ( liftM, when, guard )
 import Data.List ( nub )
 import Data.Maybe ( listToMaybe )
 import System.Directory ( removeFile, doesFileExist, getDirectoryContents )
-import System.Posix ( setFileTimes )
-import System.Posix.Files ( linkCount, modificationTime, getSymbolicLinkStatus )
-import System.Posix.Types ( EpochTime )
+import System.Posix.Files ( linkCount, getSymbolicLinkStatus )
 import System.IO ( hPutStrLn, stderr )
 
 import Crypt.SHA256 ( sha256sum )
@@ -39,7 +37,6 @@ import Darcs.Flags ( Compression( .. ) )
 import Darcs.Global ( darcsdir )
 import Darcs.Lock ( writeAtomicFilePS, gzWriteAtomicFilePS )
 import Progress ( progressList, debugMessage, debugFail )
-import Darcs.SlurpDirectory ( undefined_time )
 import Darcs.URL ( is_file )
 import Darcs.Utils ( withCurrentDirectory, catchall )
 
@@ -98,21 +95,6 @@ checkHash h s | length h == 40 = sha1PS s == h
               | otherwise = False
 
 
-findFileMtimeUsingCache :: Cache -> HashedDir -> String -> IO EpochTime
-findFileMtimeUsingCache (Ca cache) subdir f = mt cache
-    where mt [] = return undefined_time
-          mt (Cache Repo Writable r:_) = (modificationTime `fmap`
-                                          getSymbolicLinkStatus (r++"/"++darcsdir++"/"++(hashedDir subdir)++"/"++f))
-                                         `catchall` return undefined_time
-          mt (_:cs) = mt cs
-
-setFileMtimeUsingCache :: Cache -> HashedDir -> String -> EpochTime -> IO ()
-setFileMtimeUsingCache (Ca cache) subdir f t = st cache
-    where st [] = return ()
-          st (Cache Repo Writable r:_) = setFileTimes (r++"/"++darcsdir++"/"++(hashedDir subdir)++"/"++f) t t
-                                         `catchall` return ()
-          st (_:cs) = st cs
-
 fetchFileUsingCache :: Cache -> HashedDir -> String -> IO (String, B.ByteString)
 fetchFileUsingCache = fetchFileUsingCachePrivate Anywhere
 
