@@ -20,25 +20,24 @@
 #include "gadts.h"
 
 module Darcs.Repository.HashedIO ( HashedIO, applyHashed,
-                                   copyHashed, syncHashedPristine, copyPartialsHashed, listHashedContents,
+                                   copyHashed, copyPartialsHashed, listHashedContents,
                                    slurpHashedPristine, writeHashedPristine,
                                    clean_hashdir ) where
 
 import Darcs.Global ( darcsdir )
-import Data.List ( (\\) )
+import qualified Data.Set as Set
 import qualified Data.Map as Map
 import System.Directory ( getDirectoryContents, createDirectoryIfMissing )
-import System.Posix.Types ( EpochTime )
 import Control.Monad.State ( StateT, runStateT, modify, get, put, gets, lift )
 import Control.Monad ( when )
+import Control.Applicative ( (<$>) )
 import Data.Maybe ( isJust )
 import System.IO.Unsafe ( unsafeInterleaveIO )
 
 import Darcs.SlurpDirectory.Internal ( Slurpy(..), SlurpyContents(..), map_to_slurpies, slurpies_to_map )
-import Darcs.SlurpDirectory ( withSlurpy, undefined_time, undefined_size )
+import Darcs.SlurpDirectory ( withSlurpy, undefined_size )
 import Darcs.Repository.Cache ( Cache, fetchFileUsingCache, writeFileUsingCache,
                                 peekInCache, speculateFileUsingCache,
-                                findFileMtimeUsingCache, setFileMtimeUsingCache,
                                 okayHash, cleanCachesWithHint, HashedDir(..), hashedDir )
 import Darcs.Patch ( Patchy, apply )
 import Darcs.RepoPath ( FilePathLike, toFilePath )
@@ -48,7 +47,7 @@ import Darcs.Lock ( writeAtomicFilePS, removeFileMayNotExist )
 import Darcs.Utils ( withCurrentDirectory )
 import Progress ( debugMessage, beginTedious, endTedious, tediousSize, finishedOneIO, progress )
 import Darcs.Patch.FileName ( FileName, norm_path, fp2fn, fn2fp, fn2niceps, niceps2fn,
-                              break_on_dir, own_name, super_name )
+                              break_on_dir, own_name, superName )
 
 import ByteStringUtils ( linesPS, unlinesPS )
 import qualified Data.ByteString       as B  (ByteString, length, empty)
@@ -111,7 +110,7 @@ instance ReadableDirectory (HashedIO r p) where
                                                        Just h -> inh h $ mInCurrentDirectory fn'' j
         where fn' = norm_path fn
     mGetDirectoryContents = map (\ (_,f,_) -> f) `fmap` readroot
-    mReadFilePS fn = mInCurrentDirectory (super_name fn) $ do
+    mReadFilePS fn = mInCurrentDirectory (superName fn) $ do
                                           c <- readroot
                                           case geta F (own_name fn) c of
                                             Nothing -> fail $ " file don't exist... "++ fn2fp fn
@@ -170,11 +169,11 @@ identifyThing fn | fn' == fp2fn "" = do h <- gets rootHash
         where fn' = norm_path fn
 
 makeThing :: FileName -> (ObjType,String) -> HashedIO RW p ()
-makeThing fn (o,h) = mWithCurrentDirectory (super_name $ norm_path fn) $
+makeThing fn (o,h) = mWithCurrentDirectory (superName $ norm_path fn) $
                      seta o (own_name $ norm_path fn) h `fmap` readroot >>= writeroot
 
 rmThing :: FileName -> HashedIO RW p ()
-rmThing fn = mWithCurrentDirectory (super_name $ norm_path fn) $
+rmThing fn = mWithCurrentDirectory (superName $ norm_path fn) $
              do c <- readroot
                 let c' = filter (\(_,x,_)->x/= own_name (norm_path fn)) c
                 if length c' == length c - 1
@@ -191,10 +190,6 @@ readTediousHash :: String -> String -> HashedIO r p B.ByteString
 readTediousHash k h = do lift $ finishedOneIO k h
                          readhash h
 
-gethashmtime :: String -> HashedIO r p EpochTime
-gethashmtime h = do HashDir _ c _ _ <- get
-                    lift $ unsafeInterleaveIO $ findFileMtimeUsingCache c HashedPristineDir h
-
 withh :: String -> HashedIO RW p a -> HashedIO RW p (String,a)
 withh h j = do hd <- get
                put $ hd { rootHash = h }
@@ -286,10 +281,9 @@ slh = do c <- readroot
          lift $ beginTedious k
          safeInterleave $ (Slurpy rootdir . SlurpDir (Just hroot) . slurpies_to_map) `fmap` mapM sl c
     where sl (F,n,h) = do ps <- safeInterleave $ readTediousHash k h
-                          t <- gethashmtime h
                           let len = if length h == 75 then read (take 10 h)
                                                       else undefined_size
-                          return $ Slurpy n $ SlurpFile (Just h, t, len) ps
+                          return $ Slurpy n $ SlurpFile (Just h, 0, len) ps
           sl (D,n,h) = inh h $ do c <- readroot
                                   lift $ tediousSize k (length c)
                                   lift $ finishedOneIO k h
@@ -327,33 +321,6 @@ writeHashedPristine c compr sl =
                                                return (F, f, h)
           k = "Writing pristine"
 
-grab :: FileName -> Slurpy -> Maybe Slurpy
-grab _ (Slurpy _ (SlurpFile _ _)) = Nothing
-grab fn (Slurpy _ (SlurpDir _ ss)) = fmap (Slurpy fn) $ Map.lookup fn ss
-
--- | Update timestamps on pristine files to match those in the working directory
--- (which is passed to this function in form of a Slurpy). It needed for the
--- mtime-based unsafeDiff optimisation to work efficiently.
-syncHashedPristine :: Cache -> Slurpy -> String -> IO ()
-syncHashedPristine c s r = do runStateT sh $ HashDir { permissions=RW, cache=c,
-                                                       compress=compression [], rootHash=r }
-                              return ()
-    where sh = do cc <- readroot
-                  lift $ tediousSize k (length cc)
-                  mapM_ sh' cc
-          sh' (D,n,h) = case progress k $ grab n s of
-                        Just s' -> lift $ syncHashedPristine c s' h
-                        Nothing -> return ()
-          sh' (F,n,h) = case progress k $ grab n s of
-                        Just (Slurpy _ (SlurpFile (_,t',l) x)) ->
-                            do t <- lift $ findFileMtimeUsingCache c HashedPristineDir h
-                               when (t' /= undefined_time && t' /= t) $
-                                    do ps <- readhash h
-                                       when (B.length ps == fromIntegral l && ps == x) $
-                                            lift $ setFileMtimeUsingCache c HashedPristineDir h t'
-                        _ -> return ()
-          k = "Synchronizing pristine"
-
 copyHashed :: String -> Cache -> Compression -> String -> IO ()
 copyHashed k c compr z = do runStateT cph $ HashDir { permissions = RO, cache = c,
                                                       compress = compr, rootHash = z }
@@ -409,9 +376,11 @@ clean_hashdir c dir_ hashroots =
    do -- we'll remove obsolete bits of "dir"
       debugMessage $ "Cleaning out " ++ (hashedDir dir_) ++ "..."
       let hashdir = darcsdir ++ "/" ++ (hashedDir dir_) ++ "/"
-      hs <- concat `fmap` (mapM (listHashedContents "cleaning up..." c) hashroots)
-      fs <- filter okayHash `fmap` getDirectoryContents hashdir
-      mapM_ (removeFileMayNotExist . (hashdir++)) (fs \\ hs)
+      hs <- set . concat <$> mapM (listHashedContents "cleaning up..." c) hashroots
+      fs <- set . filter okayHash <$> getDirectoryContents hashdir
+      mapM_ (removeFileMayNotExist . (hashdir++)) (unset $ fs `Set.difference` hs)
       -- and also clean out any global caches.
       debugMessage "Cleaning out any global caches..."
-      cleanCachesWithHint c dir_ (fs \\ hs)
+      cleanCachesWithHint c dir_ (unset $ fs `Set.difference` hs)
+   where set = Set.fromList . map BC.pack
+         unset = map BC.unpack . Set.toList
