@@ -20,14 +20,13 @@
 #include "gadts.h"
 
 module Darcs.Repository.HashedRepo ( revert_tentative_changes, finalize_tentative_changes,
-                                     slurp_pristine, sync_repo, clean_pristine,
-                                     copy_pristine, copy_partials_pristine, pristine_from_working,
+                                     slurp_pristine, clean_pristine,
+                                     copy_pristine, copy_partials_pristine,
                                      apply_to_tentative_pristine,
-                                     replacePristineFromSlurpy,
                                      add_to_tentative_inventory, remove_from_tentative_inventory,
                                      read_repo, read_tentative_repo, write_and_read_patch,
                                      write_tentative_inventory, copy_repo, slurp_all_but_darcs,
-                                     readHashedPristineRoot
+                                     readHashedPristineRoot, pris2inv
                                    ) where
 
 import System.Directory ( doesFileExist, createDirectoryIfMissing )
@@ -45,8 +44,8 @@ import Darcs.Repository.Cache ( Cache, fetchFileUsingCache, speculateFileUsingCa
                                 unionCaches, repo2cache, okayHash, takeHash,
                                 HashedDir(..), hashedDir )
 import Darcs.Repository.HashedIO ( applyHashed, slurpHashedPristine,
-                                   copyHashed, syncHashedPristine, copyPartialsHashed,
-                                   writeHashedPristine, clean_hashdir )
+                                   copyHashed, copyPartialsHashed,
+                                   clean_hashdir )
 import Darcs.Repository.InternalTypes ( Repository(..), extractCache )
 import Darcs.Hopefully ( PatchInfoAnd, patchInfoAndPatch, n2pia, info,
                          extractHash, createHashed )
@@ -54,11 +53,11 @@ import Darcs.SlurpDirectory ( Slurpy, empty_slurpy, slurp_remove, slurp )
 import Darcs.Patch ( RepoPatch, Patchy, Named, showPatch, patch2patchinfo, readPatch )
 import Darcs.Patch.Depends ( commute_to_end, slightly_optimize_patchset )
 import Darcs.Patch.Info ( PatchInfo, showPatchInfo, human_friendly, readPatchInfo )
-import Darcs.Ordered ( unsafeCoerceP, (:<)(..) )
+import Darcs.Witnesses.Ordered ( unsafeCoerceP, (:<)(..) )
 import Darcs.Patch.FileName ( fp2fn )
 
 import ByteStringUtils ( gzReadFilePS, dropSpace )
-import qualified Data.ByteString as B (null, length, readFile, empty
+import qualified Data.ByteString as B (null, length, empty
                                       ,tail, take, drop, ByteString)
 import qualified Data.ByteString.Char8 as BC (unpack, dropWhile, break, pack)
 
@@ -69,9 +68,9 @@ import Darcs.Lock ( writeBinFile, writeDocBinFile, writeAtomicFilePS, appendBinF
 import Darcs.Utils ( withCurrentDirectory )
 import Progress ( beginTedious, tediousSize, endTedious, debugMessage, finishedOneIO )
 #include "impossible.h"
-import Darcs.Ordered ( FL(..), RL(..),
+import Darcs.Witnesses.Ordered ( FL(..), RL(..),
                              mapRL, mapFL, lengthRL )
-import Darcs.Sealed ( Sealed(..), seal, unseal )
+import Darcs.Witnesses.Sealed ( Sealed(..), seal, unseal )
 import Darcs.Global ( darcsdir )
 
 revert_tentative_changes :: IO ()
@@ -338,18 +337,6 @@ slurp_pristine_private c compr inv = case inv2pris inv of
                                     h | h == sha1PS B.empty -> return empty_slurpy
                                       | otherwise -> slurpHashedPristine c compr h
 
-pristine_from_working :: Cache -> Compression -> IO ()
-pristine_from_working c compr = do
-  s <- slurp_all_but_darcs "."
-  replacePristineFromSlurpy c compr s
-
-replacePristineFromSlurpy :: Cache -> Compression -> Slurpy -> IO ()
-replacePristineFromSlurpy c compr s = do 
-  h <- writeHashedPristine c compr s
-  let t = darcsdir++"/hashed_inventory"
-  i <- gzReadFilePS t
-  writeDocBinFile t $ pris2inv h i
-
 copy_pristine :: Cache -> Compression -> String -> String -> IO ()
 copy_pristine c compr d iname = do
     i <- fetchFilePS (d++"/"++iname) Uncachable
@@ -359,13 +346,6 @@ copy_pristine c compr d iname = do
     copyHashed k c compr $ inv2pris i
     endTedious k
 
-sync_repo :: Cache -> IO ()
-sync_repo c = do i <- B.readFile $ darcsdir++"/hashed_inventory"
-                 s <- slurp_all_but_darcs "."
-                 beginTedious "Synchronizing pristine"
-                 syncHashedPristine c s $ inv2pris i
-                 
-
 copy_partials_pristine :: FilePathLike fp =>
                           Cache -> Compression -> String -> String -> [fp] -> IO ()
 copy_partials_pristine c compr d iname fps =
