@@ -24,131 +24,122 @@
 module Darcs.Repository.Internal ( Repository(..), RepoType(..), RIO(unsafeUnRIO), ($-),
                     maybeIdentifyRepository, identifyDarcs1Repository, identifyRepositoryFor,
                     findRepository, amInRepository, amNotInRepository,
-                    slurp_pending, pristineFromWorking, revertRepositoryChanges,
+                    slurp_pending, revertRepositoryChanges,
                     slurp_recorded, slurp_recorded_and_unrecorded,
-                    read_pending, announce_merge_conflicts, setTentativePending,
+                    announce_merge_conflicts, setTentativePending,
                     check_unrecorded_conflicts,
                     withRecorded,
-                    checkPristineAgainstSlurpy,
-                    get_unrecorded, get_unrecorded_unsorted, get_unrecorded_no_look_for_adds,
-                    get_unrecorded_in_files, get_unrecorded_in_files_unsorted,
-                    read_repo, sync_repo,
+                    read_repo,
                     prefsUrl, makePatchLazy,
-                    add_to_pending,
                     withRepoLock, withRepoReadLock,
                     withRepository, withRepositoryDirectory, withGutsOf,
                     tentativelyAddPatch, tentativelyRemovePatches, tentativelyAddToPending,
+                    tentativelyAddPatch_,
                     tentativelyReplacePatches,
-                    tentativelyMergePatches, considerMergeToWorking,
                     finalizeRepositoryChanges,
                     unrevertUrl,
                     applyToWorking, patchSetToPatches,
                     createPristineDirectoryTree, createPartialsPristineDirectoryTree,
-                    replacePristineFromSlurpy,
                     optimizeInventory, cleanRepository,
                     getMarkedupFile,
                     PatchSet, SealedPatchSet,
                     setScriptsExecutable,
                     getRepository, rIO,
-                    testTentative, testRecorded
+                    testTentative, testRecorded,
+                    UpdatePristine(..), MakeChanges(..), applyToTentativePristine,
+                    make_new_pending
                   ) where
 
 import Printer ( putDocLn, (<+>), text, ($$) )
 
 import Data.Maybe ( isJust, isNothing )
-import Darcs.Repository.Prefs ( get_prefval )
-import Darcs.Resolution ( standard_resolution, external_resolution )
+import Darcs.Repository.Prefs ( getPrefval )
+import Darcs.Repository.State ( readRecorded )
+import Darcs.Repository.LowLevel ( read_pending, pendingName, readPrims, read_pendingfile )
 import System.Exit ( ExitCode(..), exitWith )
 import System.Cmd ( system )
-import Darcs.External ( backupByCopying, clonePartialsTree )
+import Darcs.External ( clonePartialsTree )
 import Darcs.IO ( runTolerantly, runSilently )
 import Darcs.Repository.Pristine ( identifyPristine, nopristine,
-                                   easyCreatePristineDirectoryTree, slurpPristine, syncPristine,
-                                   easyCreatePartialsPristineDirectoryTree,
-                                   createPristineFromWorking )
-import qualified Darcs.Repository.Pristine as Pristine ( replacePristineFromSlurpy )
-                                                         
+                                   easyCreatePristineDirectoryTree, slurpPristine,
+                                   easyCreatePartialsPristineDirectoryTree )
+
 import Data.List ( (\\) )
 import Darcs.SignalHandler ( withSignalsBlocked )
 import Darcs.Repository.Format ( RepoFormat, RepoProperty( Darcs2, HashedInventory ),
-                                 identifyRepoFormat, format_has,
-                                 write_problem, read_problem, readfrom_and_writeto_problem )
+                                 identifyRepoFormat, formatHas,
+                                 writeProblem, readProblem, readfromAndWritetoProblem )
 import System.Directory ( doesDirectoryExist, setCurrentDirectory, removeFile,
                           createDirectoryIfMissing )
 import Control.Monad ( liftM, when, unless )
 import Workaround ( getCurrentDirectory, renameFile, setExecutable )
 
 import ByteStringUtils ( gzReadFilePS )
-import qualified Data.ByteString as B (ByteString, empty, readFile, isPrefixOf)
+import qualified Data.ByteString as B ( empty, readFile, isPrefixOf )
 import qualified Data.ByteString.Char8 as BC (pack)
 
-import Darcs.Patch ( Patch, RealPatch, Effect, is_hunk, is_binary, description,
+import Darcs.Patch ( Patch, RealPatch, Effect, primIsHunk, primIsBinary, description,
 
-                     try_to_shrink, commuteFL, commute, apply_to_filepaths )
+                     tryToShrink, commuteFL, commute )
 import Darcs.Patch.Prim ( try_shrinking_inverse, Conflict )
 import Darcs.Patch.Bundle ( scan_bundle, make_bundle )
-import Darcs.Patch.FileName ( FileName, fn2fp )
-import Darcs.Patch.TouchesFiles ( choose_touching )
-import Darcs.SlurpDirectory ( Slurpy, slurp_unboring, mmap_slurp, co_slurp,
-                              slurp_has, list_slurpy_files )
+import Darcs.SlurpDirectory ( Slurpy, mmap_slurp, co_slurp, list_slurpy_files )
 import Darcs.Hopefully ( PatchInfoAnd, info, n2pia,
                          hopefully, hopefullyM )
 import Darcs.Repository.ApplyPatches ( apply_patches )
 import qualified Darcs.Repository.HashedRepo as HashedRepo
                             ( revert_tentative_changes, finalize_tentative_changes,
-                              remove_from_tentative_inventory, sync_repo,
+                              remove_from_tentative_inventory,
                               copy_pristine, copy_partials_pristine, slurp_pristine,
-                              apply_to_tentative_pristine, pristine_from_working,
+                              apply_to_tentative_pristine,
                               write_tentative_inventory, write_and_read_patch,
                               add_to_tentative_inventory,
                               read_repo, read_tentative_repo, clean_pristine,
-                              replacePristineFromSlurpy,
                               slurp_all_but_darcs )
 import qualified Darcs.Repository.DarcsRepo as DarcsRepo
-import Darcs.Flags ( DarcsFlag(AnyOrder, Boring, LookForAdds, Verbose, Quiet,
+import Darcs.Flags ( DarcsFlag(Verbose, Quiet,
                                MarkConflicts, AllowConflicts, NoUpdateWorking,
                                WorkRepoUrl, WorkRepoDir, UMask, Test, LeaveTestDir,
-                               SetScriptsExecutable, DryRun, IgnoreTimes,
-                               Summary, NoSummary),
+                               SetScriptsExecutable, DryRun ),
                      want_external_merge, compression )
-import Darcs.Ordered ( FL(..), RL(..), EqCheck(..), unsafeCoerceP,
+import Darcs.Witnesses.Ordered ( FL(..), RL(..), EqCheck(..), unsafeCoerceP,
                              (:\/:)(..), (:/\:)(..), (:>)(..),
-                             (+>+), lengthFL, nullFL,
+                             (+>+), lengthFL,
                              allFL, filterFL,
                              reverseRL, reverseFL, concatRL, mapFL,
                              mapFL_FL, concatFL )
 import Darcs.Patch ( RepoPatch, Patchy, Prim, merge,
-                     joinPatches, sort_coalesceFL,
-                     list_conflicted_files, list_touched_files,
-                     Named, patchcontents, anonymous,
+                     joinPatches,
+                     listConflictedFiles, listTouchedFiles,
+                     Named, patchcontents,
                      commuteRL, fromPrims,
                      patch2patchinfo, readPatch,
                      writePatch, effect, invert,
-                     is_addfile, is_adddir,
-                     is_setpref,
-                     apply, apply_to_slurpy,
-                     empty_markedup_file, MarkedUpFile
+                     primIsAddfile, primIsAdddir,
+                     primIsSetpref,
+                     apply, applyToSlurpy,
+                     emptyMarkedupFile, MarkedUpFile
                    )
 import Darcs.Patch.Patchy ( Invert(..) )
 import Darcs.Patch.Permutations ( commuteWhatWeCanFL, removeFL )
 import Darcs.Patch.Info ( PatchInfo )
 import Darcs.Patch.Set ( PatchSet, SealedPatchSet )
-import Darcs.Patch.Apply ( markup_file, LineMark(None) )
+import Darcs.Patch.Apply ( markupFile, LineMark(None) )
 import Darcs.Patch.Depends ( get_common_and_uncommon, deep_optimize_patchset )
-import Darcs.Diff ( unsafeDiffAtPaths, unsafeDiff )
 import Darcs.RepoPath ( FilePathLike, AbsolutePath, toFilePath,
                         ioAbsoluteOrRemote, toPath )
 import Darcs.Utils ( promptYorn, catchall, withCurrentDirectory, withUMask, nubsort )
 import Progress ( debugMessage )
 import Darcs.ProgressPatches (progressFL)
 import Darcs.URL ( is_file )
-import Darcs.Repository.Prefs ( darcsdir_filter, boring_file_filter, filetype_function,
-                                getCaches )
+import Darcs.Repository.Prefs ( getCaches )
 import Darcs.Lock ( withLock, writeDocBinFile, withDelayedDir, removeFileMayNotExist,
                     withTempDir, withPermDir )
-import Darcs.Sealed ( Sealed(Sealed), seal, FlippedSeal(FlippedSeal), flipSeal )
+import Darcs.Witnesses.Sealed ( Sealed(Sealed), seal, FlippedSeal(FlippedSeal), flipSeal )
 import Darcs.Repository.InternalTypes( Repository(..), RepoType(..) )
 import Darcs.Global ( darcsdir )
+import System.Mem( performGC )
+
 #include "impossible.h"
 
 -- | Repository IO monad.  This monad-like datatype is responsible for
@@ -206,7 +197,7 @@ maybeIdentifyRepository opts "." =
        case rf_or_e of
          Left err -> return $ Left err
          Right rf ->
-             case read_problem rf of
+             case readProblem rf of
              Just err -> return $ Left err
              Nothing -> if darcs then do pris <- identifyPristine
                                          cs <- getCaches opts here
@@ -217,7 +208,7 @@ maybeIdentifyRepository opts url' =
     rf_or_e <- identifyRepoFormat url
     case rf_or_e of
       Left e -> return $ Left e
-      Right rf -> case read_problem rf of
+      Right rf -> case readProblem rf of
                   Just err -> return $ Left err
                   Nothing ->  do cs <- getCaches opts url
                                  return $ Right $ Repo url opts rf (DarcsRepository nopristine cs)
@@ -233,7 +224,7 @@ identifyRepositoryFor :: forall p C(r u t). RepoPatch p => Repository p C(r u t)
 identifyRepositoryFor (Repo _ opts rf _) url =
     do Repo absurl _ rf_ t <- identifyDarcs1Repository opts url
        let t' = case t of DarcsRepository x c -> DarcsRepository x c
-       case readfrom_and_writeto_problem rf_ rf of
+       case readfromAndWritetoProblem rf_ rf of
          Just e -> fail $ "Incompatibility with repository " ++ url ++ ":\n" ++ e
          Nothing -> return $ Repo absurl opts rf_ t'
 
@@ -275,8 +266,12 @@ seekRepo onFail = getCurrentDirectory >>= helper where
                   else do setCurrentDirectory startpwd
                           return onFail
 
+-- The performGC in this function is a workaround for a library/GHC bug,
+-- http://hackage.haskell.org/trac/ghc/ticket/2924 -- (doesn't seem to be a
+-- problem on fast machines, but virtual ones trip this from time to time)
 amNotInRepository :: [DarcsFlag] -> IO (Either String ())
 amNotInRepository (WorkRepoDir d:_) = do createDirectoryIfMissing False d
+                                            `catchall` (performGC >> createDirectoryIfMissing False d)
                                          -- note that the above could always fail
                                          setCurrentDirectory d
                                          amNotInRepository []
@@ -300,7 +295,7 @@ slurp_pending :: RepoPatch p => Repository p C(r u t) -> IO Slurpy
 slurp_pending repo@(Repo _ _ _ rt) = do
   cur <- slurp_recorded repo
   Sealed pend <- read_pending repo
-  case apply_to_slurpy pend cur of
+  case applyToSlurpy pend cur of
     Just pendcur -> return pendcur
     Nothing -> do putStrLn "Yikes, pending has conflicts.  Renaming file as_darcs/patches/pending_buggy"
                   renameFile (pendingName rt) (pendingName rt++"_buggy")
@@ -308,7 +303,7 @@ slurp_pending repo@(Repo _ _ _ rt) = do
 
 slurp_recorded :: RepoPatch p => Repository p C(r u t) -> IO Slurpy
 slurp_recorded (Repo dir opts rf (DarcsRepository _ c))
-    | format_has HashedInventory rf =
+    | formatHas HashedInventory rf =
         HashedRepo.slurp_pristine c (compression opts) dir $ darcsdir++"/hashed_inventory"
 slurp_recorded repository@(Repo dir _ _ (DarcsRepository p _)) = do
     mc <- withCurrentDirectory dir $ slurpPristine p
@@ -323,34 +318,11 @@ slurp_recorded_and_unrecorded repo@(Repo r _ _ _) = do
   cur <- slurp_recorded repo
   Sealed pend <- read_pending repo
   withCurrentDirectory r $
-      case apply_to_slurpy pend cur of
+      case applyToSlurpy pend cur of
       Nothing -> fail "Yikes, pending has conflicts!"
       Just pendslurp -> do unrec <- co_slurp pendslurp "."
                            return (cur, unrec)
 
-pendingName :: RepoType p -> String
-pendingName (DarcsRepository _ _) = darcsdir++"/patches/pending"
-
-read_pending :: RepoPatch p => Repository p C(r u t) -> IO (Sealed (FL Prim C(r)))
-read_pending (Repo r _ _ tp) =
-    withCurrentDirectory r (read_pendingfile (pendingName tp))
-
-add_to_pending :: RepoPatch p => Repository p C(r u t) -> FL Prim C(u y) -> IO ()
-add_to_pending (Repo _ opts _ _) _ | NoUpdateWorking `elem` opts = return ()
-add_to_pending repo p =
-    do pend <- get_unrecorded repo
-       make_new_pending repo (pend +>+ p)
-
-readPrims :: B.ByteString -> Sealed (FL Prim C(x))
-readPrims s = case readPatch s :: Maybe (Sealed (Patch C(x )), B.ByteString) of
-              Nothing -> Sealed NilFL
-              Just (Sealed p,_) -> Sealed (effect p)
-
-read_pendingfile :: String -> IO (Sealed (FL Prim C(x)))
-read_pendingfile name = do
-  pend <- gzReadFilePS name `catchall` return B.empty
-  return $ readPrims pend
-
 make_new_pending :: forall p C(r u t y). RepoPatch p => Repository p C(r u t) -> FL Prim C(r y) -> IO ()
 make_new_pending (Repo _ opts _ _) _ | NoUpdateWorking `elem` opts = return ()
 make_new_pending repo@(Repo r _ _ tp) origp =
@@ -361,7 +333,7 @@ make_new_pending repo@(Repo r _ _ tp) origp =
        writeSealedPatch newname $ seal $ fromPrims $ sfp
        cur <- slurp_recorded repo
        Sealed p <- read_pendingfile newname
-       when (isNothing $ apply_to_slurpy p cur) $ do
+       when (isNothing $ applyToSlurpy p cur) $ do
          let buggyname = pendingName tp ++ "_buggy"
          renameFile newname buggyname
          bugDoc $ text "There was an attempt to write an invalid pending!"
@@ -376,113 +348,45 @@ make_new_pending repo@(Repo r _ _ tp) origp =
 sift_for_pending :: FL Prim C(x y) -> Sealed (FL Prim C(x))
 sift_for_pending simple_ps =
  let oldps = maybe simple_ps id $ try_shrinking_inverse $ crude_sift simple_ps
- in if allFL (\p -> is_addfile p || is_adddir p) $ oldps
+ in if allFL (\p -> primIsAddfile p || primIsAdddir p) $ oldps
     then seal oldps
     else fromJust $ do
       Sealed x <- return $ sfp NilFL $ reverseFL oldps
-      return (case try_to_shrink x of
+      return (case tryToShrink x of
               ps | lengthFL ps < lengthFL oldps -> sift_for_pending ps
                  | otherwise -> seal ps)
       where sfp :: FL Prim C(a b) -> RL Prim C(c a) -> Sealed (FL Prim C(c))
             sfp sofar NilRL = seal sofar
             sfp sofar (p:<:ps)
-                | is_hunk p || is_binary p
+                | primIsHunk p || primIsBinary p
                     = case commuteFL (p :> sofar) of
                       Right (sofar' :> _) -> sfp sofar' ps
                       Left _ -> sfp (p:>:sofar) ps
             sfp sofar (p:<:ps) = sfp (p:>:sofar) ps
 
-get_unrecorded_no_look_for_adds :: RepoPatch p => Repository p C(r u t) -> [FileName] -> IO (FL Prim C(r y))
-get_unrecorded_no_look_for_adds r paths = get_unrecorded_private (filter (/= LookForAdds)) r paths 
-
-get_unrecorded_unsorted :: RepoPatch p => Repository p C(r u t) -> IO (FL Prim C(r u))
-get_unrecorded_unsorted r = get_unrecorded_in_files_unsorted r []
-
-get_unrecorded :: RepoPatch p => Repository p C(r u t) -> IO (FL Prim C(r u))
-get_unrecorded r = get_unrecorded_private id r []
-
--- | Gets the unrecorded changes in the given paths in the current repository,
---   without sorting them for presentation to the user
-get_unrecorded_in_files_unsorted :: RepoPatch p => Repository p C(r u t) -> [FileName] -> IO (FL Prim C(r u))
-get_unrecorded_in_files_unsorted = get_unrecorded_private (AnyOrder:)
-
--- | Gets the unrecorded changes in the given paths in the current repository.
-get_unrecorded_in_files :: RepoPatch p => Repository p C(r u t) -> [FileName] -> IO (FL Prim C(r u))
-get_unrecorded_in_files = get_unrecorded_private id 
-
--- | The /unrecorded/ includes the pending and the working directory changes.
---   The third argument is a list of paths: if this list is [], it will diff
---   the whole repo, but if there are elements in it, the function will return
---   only changes to files under those paths. The paths must be fixed paths 
---   starting with ".", but need not yet be unique.
-get_unrecorded_private :: RepoPatch p => ([DarcsFlag]->[DarcsFlag]) -> Repository p C(r u t) -> [FileName] -> IO (FL Prim C(r y))
-get_unrecorded_private _ (Repo _ opts _ _) _
-    | NoUpdateWorking `elem` opts = return $ unsafeCoerceP NilFL
-get_unrecorded_private modopts repository@(Repo r oldopts _ _) files =
-  withCurrentDirectory r (do
-    debugMessage "Looking for unrecorded changes..."
-    cur <- slurp_pending repository
-    work <- if LookForAdds `elem` opts
-            then do nboring <- if Boring `elem` opts
-                               then return $ darcsdir_filter
-                               else boring_file_filter
-                    slurp_unboring (myfilt cur nboring) "."
-            else co_slurp cur "."
-    ftf <- filetype_function
-    Sealed pend <- read_pending repository
-    let changed_files = apply_to_filepaths pend filesFP
-        pre_changed_files = apply_to_filepaths (invert pend) filesFP
-    Sealed relevantPend <- return $ if null files
-                                      then seal pend
-                                      else choose_touching pre_changed_files pend
-    debugMessage "diffing dir..."
-    let diffs = if null files
-                  then unsafeDiff opts ftf cur work
-                  else unsafeDiffAtPaths (ignoreTimes, lookForAdds, summary) ftf cur work changed_files
-        dif = if AnyOrder `elem` opts
-                  then relevantPend +>+ diffs
-                  else sort_coalesceFL $ relevantPend +>+ diffs
-    seq dif $ debugMessage "Found unrecorded changes."
-    return dif)
-    where myfilt s nboring f = slurp_has f s || nboring [f] /= []
-          opts = modopts oldopts
-          -- NoSummary/Summary both present gives False
-          -- Just Summary gives True
-          -- Just NoSummary gives False
-          -- Neither gives False
-          summary = Summary `elem` opts && NoSummary `notElem` opts
-          lookForAdds = LookForAdds `elem` opts
-          ignoreTimes = IgnoreTimes `elem` opts
-          filesFP = map fn2fp files
-
 -- @todo: we should not have to open the result of HashedRepo and
 -- seal it.  Instead, update this function to work with type witnesses
 -- by fixing DarcsRepo to match HashedRepo in the handling of
 -- Repository state.
 read_repo :: RepoPatch p => Repository p C(r u t) -> IO (PatchSet p C(r))
 read_repo repo@(Repo r opts rf _)
-    | format_has HashedInventory rf = do ps <- HashedRepo.read_repo repo r
+    | formatHas HashedInventory rf =  do ps <- HashedRepo.read_repo repo r
                                          return ps
     | otherwise = do Sealed ps <- DarcsRepo.read_repo opts r
                      return $ unsafeCoerceP ps
 
 readTentativeRepo :: RepoPatch p => Repository p C(r u t) -> IO (PatchSet p C(t))
 readTentativeRepo repo@(Repo r opts rf _)
-    | format_has HashedInventory rf = do ps <- HashedRepo.read_tentative_repo repo r
+    | formatHas HashedInventory rf = do  ps <- HashedRepo.read_tentative_repo repo r
                                          return ps
     | otherwise = do Sealed ps <- DarcsRepo.read_tentative_repo opts r
                      return $ unsafeCoerceP ps
 
 makePatchLazy :: RepoPatch p => Repository p C(r u t) -> PatchInfoAnd p C(x y) -> IO (PatchInfoAnd p C(x y))
 makePatchLazy (Repo r opts rf (DarcsRepository _ c)) p
-    | format_has HashedInventory rf = withCurrentDirectory r $ HashedRepo.write_and_read_patch c (compression opts) p
+    | formatHas HashedInventory rf = withCurrentDirectory r $ HashedRepo.write_and_read_patch c (compression opts) p
     | otherwise = withCurrentDirectory r $ DarcsRepo.write_and_read_patch opts p
 
-sync_repo :: Repository p C(r u t) -> IO ()
-sync_repo (Repo r _ rf (DarcsRepository _ c))
-    | format_has HashedInventory rf = withCurrentDirectory r $ HashedRepo.sync_repo c
-sync_repo (Repo r _ _ (DarcsRepository p _)) = withCurrentDirectory r $ syncPristine p
-
 prefsUrl :: Repository p C(r u t) -> String
 prefsUrl (Repo r _ _ (DarcsRepository _ _)) = r ++ "/"++darcsdir++"/prefs"
 
@@ -524,91 +428,26 @@ handle_pend_for_add (Repo _ _ _ rt) p =
           fromPrims_ = fromPrims
 
 is_simple :: Prim C(x y) -> Bool
-is_simple x = is_hunk x || is_binary x || is_setpref x
+is_simple x = primIsHunk x || primIsBinary x || primIsSetpref x
 
 crude_sift :: FL Prim C(x y) -> FL Prim C(x y)
 crude_sift xs = if allFL is_simple xs then filterFL ishunkbinary xs else xs
     where ishunkbinary :: Prim C(x y) -> EqCheck C(x y)
-          ishunkbinary x | is_hunk x || is_binary x = unsafeCoerceP IsEq
+          ishunkbinary x | primIsHunk x || primIsBinary x = unsafeCoerceP IsEq
                          | otherwise = NotEq
 
 data HashedVsOld a = HvsO { old, hashed :: a }
 
 decideHashedOrNormal :: Monad m => RepoFormat -> HashedVsOld (m a) -> m a
 decideHashedOrNormal rf (HvsO { hashed = h, old = o })
-    | format_has HashedInventory rf = h
+    | formatHas HashedInventory rf = h
     | otherwise = o
 
-
-tentativelyMergePatches :: RepoPatch p
-                        => Repository p C(r u t) -> String -> [DarcsFlag]
-                        -> FL (PatchInfoAnd p) C(x r) -> FL (PatchInfoAnd p) C(x y)
-                        -> IO (Sealed (FL Prim C(u)))
-tentativelyMergePatches = tentativelyMergePatches_ MakeChanges
-
-considerMergeToWorking :: RepoPatch p
-                       => Repository p C(r u t) -> String -> [DarcsFlag]
-                       -> FL (PatchInfoAnd p) C(x r) -> FL (PatchInfoAnd p) C(x y)
-                       -> IO (Sealed (FL Prim C(u)))
-considerMergeToWorking = tentativelyMergePatches_ DontMakeChanges
-
 data MakeChanges = MakeChanges | DontMakeChanges deriving ( Eq )
 
-tentativelyMergePatches_ :: forall p C(r u t y x). RepoPatch p
-                         => MakeChanges
-                         -> Repository p C(r u t) -> String -> [DarcsFlag]
-                         -> FL (PatchInfoAnd p) C(x r) -> FL (PatchInfoAnd p) C(x y)
-                         -> IO (Sealed (FL Prim C(u)))
-tentativelyMergePatches_ mc r cmd opts usi themi =
-  do let us = mapFL_FL hopefully usi
-         them = mapFL_FL hopefully themi
-     _ :/\: pc <- return $ merge (progressFL "Merging them" them :\/: progressFL "Merging us" us)
-     pend <- get_unrecorded_unsorted r -- we don't care if it looks pretty...
-     anonpend <- anonymous (fromPrims pend)
-     pend' :/\: pw <- return $ merge (pc :\/: anonpend :>: NilFL)
-     let pwprim = joinPatches $ progressFL "Examining patches for conflicts" $ mapFL_FL patchcontents pw
-     Sealed standard_resolved_pw <- return $ standard_resolution pwprim
-     debugMessage "Checking for conflicts..."
-     mapM_ backupByCopying $ list_touched_files standard_resolved_pw
-     debugMessage "Announcing conflicts..."
-     have_conflicts <- announce_merge_conflicts cmd opts standard_resolved_pw
-     debugMessage "Checking for unrecorded conflicts..."
-     have_unrecorded_conflicts <- check_unrecorded_conflicts opts pc
-     debugMessage "Reading working directory..."
-     (_, working) <- slurp_recorded_and_unrecorded r
-     debugMessage "Working out conflicts in actual working directory..."
-     Sealed pw_resolution <-
-          case (want_external_merge opts, have_conflicts || have_unrecorded_conflicts) of
-          (Nothing,_) -> return $ if AllowConflicts `elem` opts
-                                  then seal NilFL
-                                  else seal standard_resolved_pw
-          (_,False) -> return $ seal standard_resolved_pw
-          (Just c, True) -> external_resolution working c
-                                                    (effect us +>+ pend)
-                                                    (effect them) pwprim
-     debugMessage "Applying patches to the local directories..."
-     when (mc == MakeChanges) $
-          do let doChanges :: FL (PatchInfoAnd p) C(x r) -> IO ()
-                 doChanges NilFL = applyps r themi
-                 doChanges _     = applyps r (mapFL_FL n2pia pc)
-             doChanges usi
-             setTentativePending r (effect pend' +>+ pw_resolution)
-     return $ seal (effect pwprim +>+ pw_resolution)
-  where mapAdd :: Repository p C(i l m) -> FL (PatchInfoAnd p) C(i j) -> [IO ()]
-        mapAdd _ NilFL = []
-        mapAdd r'@(Repo dir df rf dr) (a:>:as) =
-               -- we construct a new Repository object on the recursive case so that the
-               -- recordedstate of the repository can match the fact that we just wrote a patch
-               tentativelyAddPatch_ DontUpdatePristine r' opts a : mapAdd (Repo dir df rf dr) as
-        applyps :: Repository p C(i l m) -> FL (PatchInfoAnd p) C(i j) -> IO ()
-        applyps repo ps = do debugMessage "Adding patches to inventory..."
-                             sequence_ $ mapAdd repo ps
-                             debugMessage "Applying patches to pristine..."
-                             applyToTentativePristine repo ps
-
 announce_merge_conflicts :: String -> [DarcsFlag] -> FL Prim C(x y) -> IO Bool
 announce_merge_conflicts cmd opts resolved_pw =
-    case nubsort $ list_touched_files $ resolved_pw of
+    case nubsort $ listTouchedFiles $ resolved_pw of
     [] -> return False
     cfs -> if MarkConflicts `elem` opts || AllowConflicts `elem` opts
               || want_external_merge opts /= Nothing
@@ -636,11 +475,11 @@ check_unrecorded_conflicts opts pc =
                        pend ->
                            case merge (fromPrims_ pend :\/: fromPrims_ (concatFL $ mapFL_FL effect pc)) of
                            _ :/\: pend' ->
-                               case list_conflicted_files pend' of
+                               case listConflictedFiles pend' of
                                [] -> return False
-                               fs -> do yorn <- promptYorn
-                                                ("You have conflicting local changes to:\n"
-                                                 ++ unwords fs++"\nProceed?")
+                               fs -> do putStrLn ("You have conflicting local changes to:\n"
+                                                 ++ unwords fs)
+                                        yorn <- promptYorn "Proceed?"
                                         when (yorn /= 'y') $
                                              do putStrLn "Cancelled."
                                                 exitWith ExitSuccess
@@ -736,7 +575,7 @@ tentativelyRemovePatches_ up repository@(Repo dir _ rf (DarcsRepository _ c)) op
                                        prepend repository $ effect ps
       remove_from_unrevert_context repository ps
       debugMessage "Removing changes from tentative inventory..."
-      if format_has HashedInventory rf
+      if formatHas HashedInventory rf
         then do HashedRepo.remove_from_tentative_inventory repository (compression opts) ps
                 when (up == UpdatePristine) $
                      HashedRepo.apply_to_tentative_pristine c opts $
@@ -773,7 +612,7 @@ finalizeRepositoryChanges :: RepoPatch p => Repository p C(r u t) -> IO ()
 finalizeRepositoryChanges (Repo _ opts _ _)
     | DryRun `elem` opts = bug "finalizeRepositoryChanges called when --dry-run specified"
 finalizeRepositoryChanges repository@(Repo dir opts rf _)
-    | format_has HashedInventory rf =
+    | formatHas HashedInventory rf =
         withCurrentDirectory dir $ do debugMessage "Considering whether to test..."
                                       testTentative repository
                                       debugMessage "Finalizing changes..."
@@ -802,7 +641,7 @@ testAny withD repository@(Repo dir opts _ _) =
     when (Test `elem` opts) $ withCurrentDirectory dir $
     do let putInfo = if not $ Quiet `elem` opts then putStrLn else const (return ())
        debugMessage "About to run test if it exists."
-       testline <- get_prefval "test"
+       testline <- getPrefval "test"
        case testline of
          Nothing -> return ()
          Just testcode ->
@@ -837,7 +676,7 @@ getUMask ((UMask u):_) = Just u
 getUMask (_:l) = getUMask l
 
 withGutsOf :: Repository p C(r u t) -> IO () -> IO ()
-withGutsOf (Repo _ _ rf _) | format_has HashedInventory rf = id
+withGutsOf (Repo _ _ rf _) | formatHas HashedInventory rf = id
                            | otherwise = withSignalsBlocked
 
 withRepository :: [DarcsFlag] -> (forall p C(r u). RepoPatch p => Repository p C(r u r) -> IO a) -> IO a
@@ -848,7 +687,7 @@ withRepositoryDirectory :: forall a. [DarcsFlag] -> String
 withRepositoryDirectory opts1 url job =
     do Repo dir opts rf rt <- identifyDarcs1Repository opts1 url
        let rt' = case rt of DarcsRepository t c -> DarcsRepository t c
-       if format_has Darcs2 rf
+       if formatHas Darcs2 rf
          then do debugMessage $ "Identified darcs-2 repo: " ++ dir
                  job1_ (Repo dir opts rf rt')
          else do debugMessage $ "Identified darcs-1 repo: " ++ dir
@@ -868,7 +707,7 @@ x $- y = x y
 withRepoLock :: [DarcsFlag] -> (forall p C(r u). RepoPatch p => Repository p C(r u r) -> IO a) -> IO a
 withRepoLock opts job =
     withRepository opts $- \repository@(Repo _ _ rf _) ->
-    do case write_problem rf of
+    do case writeProblem rf of
          Nothing -> return ()
          Just err -> fail err
        let name = "./"++darcsdir++"/lock"
@@ -882,13 +721,13 @@ withRepoLock opts job =
 withRepoReadLock :: [DarcsFlag] -> (forall p C(r u). RepoPatch p => Repository p C(r u r) -> IO a) -> IO a
 withRepoReadLock opts job =
     withRepository opts $- \repository@(Repo _ _ rf _) ->
-    do case write_problem rf of
+    do case writeProblem rf of
          Nothing -> return ()
          Just err -> fail err
        let name = "./"++darcsdir++"/lock"
            wu = case (getUMask opts) of Nothing -> id
                                         Just u -> withUMask u
-       wu $ if format_has HashedInventory rf || DryRun `elem` opts
+       wu $ if formatHas HashedInventory rf || DryRun `elem` opts
             then job repository
             else withLock name (revertRepositoryChanges repository >> job repository)
 
@@ -898,7 +737,8 @@ remove_from_unrevert_context repository ps = do
   Sealed bundle <- unrevert_patch_bundle `catchall` (return $ seal (NilRL:<:NilRL))
   remove_from_unrevert_context_ bundle
   where unrevert_impossible unrevert_loc =
-            do yorn <- promptYorn "This operation will make unrevert impossible!\nProceed?"
+            do putStrLn "This operation will make unrevert impossible!"
+               yorn <- promptYorn "Proceed?"
                case yorn of
                  'n' -> fail "Cancelled."
                  'y' -> removeFile unrevert_loc `catchall` return ()
@@ -916,15 +756,14 @@ remove_from_unrevert_context repository ps = do
             debugMessage "Adjusting the context of the unrevert changes..."
             ref <- readTentativeRepo repository
             case get_common_and_uncommon (bundle, ref) of
-                 (common,(h_us:<:NilRL):<:NilRL :\/: NilRL:<:NilRL) ->
+                 (common,(h_us:<:NilRL) :\/: NilRL) ->
                     case commuteRL (reverseFL ps :> hopefully h_us) of
                     Nothing -> unrevert_impossible unrevert_loc
                     Just (us' :> _) -> do
-                        s <- slurp_recorded repository
-                        writeDocBinFile unrevert_loc $
-                             make_bundle [] s
-                             (common \\ pis) (us':>:NilFL)
-                 (common,(x:<:NilRL):<:NilRL:\/:_)
+                        s <- readRecorded repository
+                        bundle' <- make_bundle [] s (common \\ pis) (us' :>: NilFL)
+                        writeDocBinFile unrevert_loc bundle'
+                 (common,(x:<:NilRL):\/:_)
                         | isr && any (`elem` common) pis -> unrevert_impossible unrevert_loc
                         | isr -> return ()
                         where isr = isJust $ hopefullyM x
@@ -957,14 +796,9 @@ cleanRepository repository@(Repo _ _ rf _) =
     HvsO { hashed = HashedRepo.clean_pristine repository,
            old = return () }
 
-replacePristineFromSlurpy :: Repository p C(r u t) -> Slurpy -> IO ()
-replacePristineFromSlurpy (Repo r opts rf (DarcsRepository pris c)) s
-    | format_has HashedInventory rf = withCurrentDirectory r $ HashedRepo.replacePristineFromSlurpy c (compression opts) s
-    | otherwise = withCurrentDirectory r $ Pristine.replacePristineFromSlurpy s pris
-
 createPristineDirectoryTree :: RepoPatch p => Repository p C(r u t) -> FilePath -> IO ()
 createPristineDirectoryTree repo@(Repo r opts rf (DarcsRepository pris c)) reldir
-    | format_has HashedInventory rf =
+    | formatHas HashedInventory rf =
         do createDirectoryIfMissing True reldir
            withCurrentDirectory reldir $ HashedRepo.copy_pristine c (compression opts) r (darcsdir++"/hashed_inventory")
     | otherwise =
@@ -977,7 +811,7 @@ createPristineDirectoryTree repo@(Repo r opts rf (DarcsRepository pris c)) reldi
 -- fp below really should be FileName
 createPartialsPristineDirectoryTree :: (FilePathLike fp, RepoPatch p) => Repository p C(r u t) -> [fp] -> FilePath -> IO ()
 createPartialsPristineDirectoryTree (Repo r opts rf (DarcsRepository _ c)) prefs dir
-    | format_has HashedInventory rf =
+    | formatHas HashedInventory rf =
         do createDirectoryIfMissing True dir
            withCurrentDirectory dir $
                HashedRepo.copy_partials_pristine c (compression opts) r (darcsdir++"/hashed_inventory") prefs
@@ -987,34 +821,17 @@ createPartialsPristineDirectoryTree r@(Repo rdir _ _ (DarcsRepository pris _)) p
       unless done $ withRecorded r (withTempDir "recorded") $ \_ -> do
           clonePartialsTree "." dir (map toFilePath prefs)
 
-pristineFromWorking :: RepoPatch p => Repository p C(r u t) -> IO ()
-pristineFromWorking (Repo dir opts rf (DarcsRepository _ c))
-    | format_has HashedInventory rf =
-        withCurrentDirectory dir $ HashedRepo.pristine_from_working c (compression opts)
-pristineFromWorking (Repo dir _ _ (DarcsRepository p _)) =
-  withCurrentDirectory dir $ createPristineFromWorking p
-
 withRecorded :: RepoPatch p => Repository p C(r u t)
              -> ((AbsolutePath -> IO a) -> IO a) -> (AbsolutePath -> IO a) -> IO a
 withRecorded repository mk_dir f
     = mk_dir $ \d -> do createPristineDirectoryTree repository (toFilePath d)
                         f d
 
-checkPristineAgainstSlurpy :: RepoPatch p => Repository p C(r u t) -> Slurpy -> IO Bool
-checkPristineAgainstSlurpy repository@(Repo _ opts _ _) s2 =
-    do s1 <- slurp_recorded repository
-       ftf <- filetype_function
-       -- The @$!@ is necessary because some code called from this function uses
-       -- unsafeInterleaveIO around functions that throw exceptions. If one used
-       -- @$@ instead of @$!@ here, those exceptions might not be caught by code
-       -- that runs this function inside a @catch@.
-       return $! nullFL $ unsafeDiff (LookForAdds:IgnoreTimes:opts) ftf s1 s2
-
 withTentative :: forall p a C(r u t). RepoPatch p =>
                  Repository p C(r u t) -> ((AbsolutePath -> IO a) -> IO a)
               -> (AbsolutePath -> IO a) -> IO a
 withTentative (Repo dir opts rf (DarcsRepository _ c)) mk_dir f
-    | format_has HashedInventory rf =
+    | formatHas HashedInventory rf =
         mk_dir $ \d -> do HashedRepo.copy_pristine c (compression opts) dir (darcsdir++"/tentative_pristine")
                           f d
 withTentative repository@(Repo dir opts _ _) mk_dir f =
@@ -1032,7 +849,7 @@ getMarkedupFile :: RepoPatch p => Repository p C(r u t) -> PatchInfo -> FilePath
 getMarkedupFile repository pinfo f = do
   Sealed (FlippedSeal patches) <- (seal . dropWhileFL ((/= pinfo) . info)
                                   . reverseRL . concatRL) `liftM` read_repo repository
-  return $ snd $ do_mark_all patches (f, empty_markedup_file)
+  return $ snd $ do_mark_all patches (f, emptyMarkedupFile)
   where dropWhileFL :: (FORALL(x y) a C(x y) -> Bool) -> FL a C(r v) -> FlippedSeal (FL a) C(v)
         dropWhileFL _ NilFL       = flipSeal NilFL
         dropWhileFL p xs@(x:>:xs')
@@ -1042,7 +859,7 @@ do_mark_all :: RepoPatch p => FL (PatchInfoAnd p) C(x y)
             -> (FilePath, MarkedUpFile) -> (FilePath, MarkedUpFile)
 do_mark_all (hp:>:pps) (f, mk) =
     case hopefullyM hp of
-    Just p -> do_mark_all pps $ markup_file (info hp) (patchcontents p) (f, mk)
+    Just p -> do_mark_all pps $ markupFile (info hp) (patchcontents p) (f, mk)
     Nothing -> (f, [(BC.pack "Error reading a patch!",None)])
 do_mark_all NilFL (f, mk) = (f, mk)
 
