@@ -33,6 +33,7 @@ module Darcs.SelectChanges ( with_selected_changes',
                        with_selected_last_changes_reversed,
                        view_changes,
                        with_selected_patch_from_repo,
+                       filterOutConflicts,
                      ) where
 import System.IO
 import Data.List ( intersperse )
@@ -43,33 +44,38 @@ import System.Exit ( exitWith, ExitCode(ExitSuccess) )
 
 import English ( Noun(..), englishNum  )
 import Darcs.Arguments ( showFriendly )
-import Darcs.Hopefully ( PatchInfoAnd, hopefully )
-import Darcs.Repository ( Repository, read_repo )
+import Darcs.Hopefully ( PatchInfoAnd, hopefully, n2pia )
+import Darcs.Repository ( Repository, read_repo, unrecordedChanges )
 import Darcs.Patch ( RepoPatch, Patchy, Prim, summary,
-                     invert, list_touched_files,
-                     commuteFL )
+                     invert, listTouchedFiles,
+                     commuteFL, fromPrims, anonymous )
 import qualified Darcs.Patch ( thing, things )
-import Darcs.Ordered ( FL(..), RL(..), (:>)(..),
+import Darcs.Patch.Split ( Splitter(..) )
+import Darcs.Witnesses.Ordered ( FL(..), RL(..), (:>)(..), (:||:)(..),
                        (+>+), lengthFL, lengthRL, concatRL, mapFL_FL,
                        spanFL, reverseFL, (+<+), mapFL,
                        unsafeCoerceP )
-import Darcs.Patch.Choices ( PatchChoices, patch_choices, patch_choices_tps,
-                             force_first, force_last, make_uncertain, tag,
-                      get_choices,
-                      separate_first_middle_from_last,
-                      separate_first_from_middle_last,
-                      patch_slot,
-                      select_all_middles,
-                      force_matching_last,
-                      force_matching_first, make_everything_later,
-                             TaggedPatch, tp_patch, Slot(..),
+import Darcs.Patch.Choices ( PatchChoices, patchChoices, patchChoicesTps,
+                             patchChoicesTpsSub,
+                             forceFirst, forceLast, makeUncertain, tag,
+                      getChoices,
+                      separateFirstMiddleFromLast,
+                      separateFirstFromMiddleLast,
+                      patchSlot,
+                      selectAllMiddles,
+                      forceMatchingLast,
+                      forceMatchingFirst, makeEverythingLater,
+                             TaggedPatch, tpPatch, Slot(..),
+                      substitute,
                     )
+import Darcs.Patch.Permutations ( partitionConflictingFL, selfCommuter, commuterIdRL )
 import Darcs.Patch.TouchesFiles ( deselect_not_touching, select_not_touching )
 import Darcs.PrintPatch ( printFriendly, printPatch, printPatchPager )
-import Darcs.Match ( have_nonrange_match, match_a_patch, match_a_patchread )
-import Darcs.Flags ( DarcsFlag( Summary, DontGrabDeps, Verbose, DontPromptForDependencies), isInteractive )
-import Darcs.Sealed ( FlippedSeal(..), flipSeal, seal2, unseal2 )
+import Darcs.Match ( haveNonrangeMatch, matchAPatch, matchAPatchread )
+import Darcs.Flags ( DarcsFlag( Summary, DontGrabDeps, Verbose, DontPromptForDependencies, SkipConflicts), isInteractive )
+import Darcs.Witnesses.Sealed ( FlippedSeal(..), flipSeal, seal2, unseal2, Sealed(..) )
 import Darcs.Utils ( askUser, promptCharFancy )
+import Darcs.Lock ( editText )
 import Printer ( prefix, putDocLn )
 #include "impossible.h"
 
@@ -80,6 +86,7 @@ type MatchCriterion p = FORALL(u v) WhichChanges -> [DarcsFlag] -> (p C(u v)) ->
 type WithPatches p a C(x y) =
         String              -- jobname
      -> [DarcsFlag]         -- opts
+     -> Maybe (Splitter p)  -- for interactive editing
      -> FL p C(x y)         -- patches to select among
      -> ((FL p :> FL p) C(x y) -> IO a) -- job
      -> IO a                -- result of running job
@@ -88,6 +95,7 @@ type WithPatches p a C(x y) =
 type WithPatchesToFiles p a C(x y) =
         String              -- jobname
      -> [DarcsFlag]         -- opts
+     -> Maybe (Splitter p)  -- for interactive editing
      -> [FilePath]          -- files
      -> FL p C(x y)         -- patches to select among
      -> ((FL p :> FL p) C(x y) -> IO a) -- job
@@ -107,8 +115,8 @@ triv :: MatchCriterion p
 triv _ _ _ = True
 
 iswanted :: Patchy p => MatchCriterion (PatchInfoAnd p)
-iswanted First opts p = match_a_patch opts . hopefully $ p
-iswanted LastReversed opts p = match_a_patch opts . hopefully . invert $ p
+iswanted First opts p = matchAPatch opts . hopefully $ p
+iswanted LastReversed opts p = matchAPatch opts . hopefully . invert $ p
 iswanted Last _ _ = bug "don't support patch matching with Last in wasp"
 iswanted FirstReversed _ _ = bug "don't support patch matching with FirstReversed in wasp"
 
@@ -133,34 +141,34 @@ with_selected_last_changes_reversed = wasc LastReversed iswanted
 
 -- | wasc and wasc_ are just shorthand for with_any_selected_changes
 wasc  :: Patchy p => WhichChanges -> MatchCriterion p -> WithPatches p a C(x y)
-wasc mwch crit j o = wasc_ mwch crit j o []
+wasc mwch crit j o spl = wasc_ mwch crit j o spl []
 wasc_ :: Patchy p => WhichChanges -> MatchCriterion p -> WithPatchesToFiles p a C(x y)
 wasc_ = with_any_selected_changes
 
 with_any_selected_changes :: Patchy p => WhichChanges -> MatchCriterion p -> WithPatchesToFiles p a C(x y)
-with_any_selected_changes Last crit jn opts fs =
+with_any_selected_changes Last crit jn opts splitter fs =
     with_any_selected_changes_last
         (patches_to_consider_last' fs opts crit)
-        crit jn opts fs
-with_any_selected_changes First crit jn opts fs =
+        crit jn opts splitter fs
+with_any_selected_changes First crit jn opts splitter fs =
     with_any_selected_changes_first
        (patches_to_consider_first' fs opts crit)
-       crit jn opts fs
-with_any_selected_changes FirstReversed crit jn opts fs =
+       crit jn opts splitter fs
+with_any_selected_changes FirstReversed crit jn opts splitter fs =
     with_any_selected_changes_first_reversed
        (patches_to_consider_first_reversed' fs opts crit)
-       crit jn opts fs
-with_any_selected_changes LastReversed crit jn opts fs =
+       crit jn opts splitter fs
+with_any_selected_changes LastReversed crit jn opts splitter fs =
     with_any_selected_changes_last_reversed
         (patches_to_consider_last_reversed' fs opts crit)
-        crit jn opts fs
+        crit jn opts splitter fs
 
 
 view_changes :: RepoPatch p => [DarcsFlag] -> FL (PatchInfoAnd p) C(x y) -> IO ()
 view_changes opts ps = do
   text_view opts Nothing 0 NilRL init_tps init_pc
   return ()
-    where (init_pc, init_tps) = patch_choices_tps ps
+    where (init_pc, init_tps) = patchChoicesTps ps
 
 data KeyPress a = KeyPress { kp     :: Char
                            , kpHelp :: String }
@@ -183,7 +191,7 @@ with_selected_patch_from_repo :: forall p C(r u t). RepoPatch p => String -> Rep
                               -> (FORALL(a) (FL (PatchInfoAnd p) :> PatchInfoAnd p) C(a r) -> IO ()) -> IO ()
 with_selected_patch_from_repo jn repository opts job = do
     p_s <- read_repo repository
-    sp <- wspfr jn (match_a_patchread opts)
+    sp <- wspfr jn (matchAPatchread opts)
                               (concatRL p_s) NilFL
     case sp of
      Just (FlippedSeal (skipped :> selected)) -> job (skipped :> selected)
@@ -228,69 +236,76 @@ wspfr jn matches (p:<:pps) skipped
                   repeat_this
   where jn_cap = (toUpper $ head jn) : tail jn
 
+-- After selecting with a splitter, the results may not be canonical
+canonizeWith :: Maybe (Splitter p) -> (FL p :> FL p) C(x y) -> (FL p :> FL p) C(x y)
+canonizeWith Nothing xy = xy
+canonizeWith (Just spl) (x :> y) = canonizeSplit spl x :> canonizeSplit spl y
+
 with_any_selected_changes_last :: forall p a C(x y). Patchy p
                                => (FL p C(x y) -> (FL p :> FL p) C(x y))
                                -> MatchCriterion p
                                -> WithPatchesToFiles p a C(x y)
-with_any_selected_changes_last p2c crit jobname opts _ ps job =
+with_any_selected_changes_last p2c crit jobname opts splitter _ ps job =
  case p2c ps of
  ps_to_consider :> other_ps ->
          if not $ isInteractive opts
          then job $ ps_to_consider :> other_ps
-         else do pc <- tentatively_text_select "" jobname (Noun "patch") Last crit
+         else do pc <- tentatively_text_select splitter "" jobname (Noun "patch") Last crit
                                               opts ps_len 0 NilRL init_tps init_pc
-                 job $ selected_patches_last rejected_ps pc
+                 job $ canonizeWith splitter $ selected_patches_last rejected_ps pc
          where rejected_ps = ps_to_consider
                ps_len = lengthFL init_tps
-               (init_pc, init_tps) = patch_choices_tps $ other_ps
+               (init_pc, init_tps) = patchChoicesTps $ other_ps
 
 with_any_selected_changes_first :: forall p a C(x y). Patchy p
                                 => (FL p C(x y) -> (FL p :> FL p) C(x y))
                                 -> MatchCriterion p
                                 -> WithPatchesToFiles p a C(x y)
-with_any_selected_changes_first p2c crit jobname opts _ ps job =
+with_any_selected_changes_first p2c crit jobname opts splitter _ ps job =
  case p2c ps of
  ps_to_consider :> other_ps ->
          if not $ isInteractive opts
          then job $ ps_to_consider :> other_ps
-         else do pc <- tentatively_text_select "" jobname (Noun "patch") First crit
+         else do pc <- tentatively_text_select splitter "" jobname (Noun "patch") First crit
                                               opts ps_len 0 NilRL init_tps init_pc
-                 job $ selected_patches_first rejected_ps pc
+                 job $ canonizeWith splitter $ selected_patches_first rejected_ps pc
          where rejected_ps = other_ps
                ps_len = lengthFL init_tps
-               (init_pc, init_tps) = patch_choices_tps $ ps_to_consider
+               (init_pc, init_tps) = patchChoicesTps $ ps_to_consider
 
 with_any_selected_changes_first_reversed :: forall p a C(x y). Patchy p
                                 => (FL p C(x y) -> (FL p :> FL p) C(y x))
                                 -> MatchCriterion p
                                 -> WithPatchesToFiles p a C(x y)
-with_any_selected_changes_first_reversed p2c crit jobname opts _ ps job =
+with_any_selected_changes_first_reversed p2c crit jobname opts splitter _ ps job =
  case p2c ps of
  ps_to_consider :> other_ps ->
          if not $ isInteractive opts
          then job $ invert other_ps :> invert ps_to_consider
-         else do pc <- tentatively_text_select "" jobname (Noun "patch") FirstReversed crit
+         else do pc <- tentatively_text_select splitter
+                                             "" jobname (Noun "patch") FirstReversed crit
                                              opts ps_len 0 NilRL init_tps init_pc
-                 job $ selected_patches_first_reversed rejected_ps pc
+                 job $ canonizeWith splitter $ selected_patches_first_reversed rejected_ps pc
          where rejected_ps = ps_to_consider
                ps_len = lengthFL init_tps
-               (init_pc, init_tps) = patch_choices_tps other_ps
+               (init_pc, init_tps) = patchChoicesTps other_ps
 
 with_any_selected_changes_last_reversed :: forall p a C(x y). Patchy p
                                 => (FL p C(x y) -> (FL p :> FL p) C(y x))
                                 -> MatchCriterion p
                                 -> WithPatchesToFiles p a C(x y)
-with_any_selected_changes_last_reversed p2c crit jobname opts _ ps job =
+with_any_selected_changes_last_reversed p2c crit jobname opts splitter _ ps job =
  case p2c ps of
  ps_to_consider :> other_ps ->
          if not $ isInteractive opts
          then job $ invert other_ps :> invert ps_to_consider
-         else do pc <- tentatively_text_select "" jobname (Noun "patch") LastReversed crit
+         else do pc <- tentatively_text_select splitter
+                                             "" jobname (Noun "patch") LastReversed crit
                                              opts ps_len 0 NilRL init_tps init_pc
-                 job $ selected_patches_last_reversed rejected_ps pc
+                 job $ canonizeWith splitter $ selected_patches_last_reversed rejected_ps pc
          where rejected_ps = other_ps
                ps_len = lengthFL init_tps
-               (init_pc, init_tps) = patch_choices_tps ps_to_consider
+               (init_pc, init_tps) = patchChoicesTps ps_to_consider
 
 
 patches_to_consider_first' :: Patchy p
@@ -301,16 +316,16 @@ patches_to_consider_first' :: Patchy p
                      -> (FL p :> FL p) C(x y)
 patches_to_consider_first' fs opts crit ps =
   let deselect_unwanted pc =
-        if have_nonrange_match opts
+        if haveNonrangeMatch opts
         then if DontGrabDeps `elem` opts
-                  then force_matching_last (not.iswanted_) pc
-                  else make_everything_later $ force_matching_first iswanted_ pc
+                  then forceMatchingLast (not.iswanted_) pc
+                  else makeEverythingLater $ forceMatchingFirst iswanted_ pc
         else pc
-      iswanted_ = crit First opts . tp_patch
-  in if null fs && not (have_nonrange_match opts)
+      iswanted_ = crit First opts . tpPatch
+  in if null fs && not (haveNonrangeMatch opts)
      then ps :> NilFL
-     else tp_patches $ separate_first_middle_from_last $ deselect_not_touching fs
-                     $ deselect_unwanted $ patch_choices ps
+     else tp_patches $ separateFirstMiddleFromLast $ deselect_not_touching fs
+                     $ deselect_unwanted $ patchChoices ps
 
 patches_to_consider_last' :: Patchy p
                      => [FilePath]  -- ^ files
@@ -320,15 +335,15 @@ patches_to_consider_last' :: Patchy p
                      -> (FL p :> FL p) C(x y)
 patches_to_consider_last' fs opts crit ps =
   let deselect_unwanted pc =
-        if have_nonrange_match opts
+        if haveNonrangeMatch opts
         then if DontGrabDeps `elem` opts
-                  then force_matching_last (not.iswanted_) pc
-                  else make_everything_later $ force_matching_first iswanted_ pc
+                  then forceMatchingLast (not.iswanted_) pc
+                  else makeEverythingLater $ forceMatchingFirst iswanted_ pc
         else pc
-      iswanted_ = crit Last opts . tp_patch
-  in if null fs && not (have_nonrange_match opts)
+      iswanted_ = crit Last opts . tpPatch
+  in if null fs && not (haveNonrangeMatch opts)
      then NilFL :> ps
-     else case get_choices $ select_not_touching fs $ deselect_unwanted $ patch_choices ps of
+     else case getChoices $ select_not_touching fs $ deselect_unwanted $ patchChoices ps of
            fc :> mc :> lc -> tp_patches $ fc :> mc +>+ lc
 
 patches_to_consider_first_reversed' :: Patchy p
@@ -339,15 +354,15 @@ patches_to_consider_first_reversed' :: Patchy p
                      -> (FL p :> FL p) C(y x)
 patches_to_consider_first_reversed' fs opts crit ps =
   let deselect_unwanted pc =
-        if have_nonrange_match opts
+        if haveNonrangeMatch opts
         then if DontGrabDeps `elem` opts
-                  then force_matching_last (not.iswanted_) pc
-                  else make_everything_later $ force_matching_first iswanted_ pc
+                  then forceMatchingLast (not.iswanted_) pc
+                  else makeEverythingLater $ forceMatchingFirst iswanted_ pc
         else pc
-      iswanted_ = crit FirstReversed opts . tp_patch
-  in if null fs && not (have_nonrange_match opts)
+      iswanted_ = crit FirstReversed opts . tpPatch
+  in if null fs && not (haveNonrangeMatch opts)
      then NilFL :> (invert ps)
-     else case get_choices $ select_not_touching fs $ deselect_unwanted $ patch_choices $ invert ps of
+     else case getChoices $ select_not_touching fs $ deselect_unwanted $ patchChoices $ invert ps of
            fc :> mc :> lc -> tp_patches $ fc :> mc +>+ lc
 
 patches_to_consider_last_reversed' :: Patchy p
@@ -358,55 +373,55 @@ patches_to_consider_last_reversed' :: Patchy p
                      -> (FL p :> FL p) C(y x)
 patches_to_consider_last_reversed' fs opts crit ps =
   let deselect_unwanted pc =
-        if have_nonrange_match opts
+        if haveNonrangeMatch opts
         then if DontGrabDeps `elem` opts
-             then force_matching_last (not.iswanted_) pc
-             else make_everything_later $ force_matching_first iswanted_ pc
+             then forceMatchingLast (not.iswanted_) pc
+             else makeEverythingLater $ forceMatchingFirst iswanted_ pc
         else pc
-      iswanted_ = crit LastReversed opts . tp_patch
+      iswanted_ = crit LastReversed opts . tpPatch
   in
-    if null fs && not (have_nonrange_match opts)
+    if null fs && not (haveNonrangeMatch opts)
     then (invert ps) :> NilFL
-    else tp_patches $ separate_first_middle_from_last $ deselect_not_touching fs
-                     $ deselect_unwanted $ patch_choices $ invert ps
+    else tp_patches $ separateFirstMiddleFromLast $ deselect_not_touching fs
+                     $ deselect_unwanted $ patchChoices $ invert ps
 
 -- | Returns the results of a patch selection user interaction
 selected_patches_last :: Patchy p => FL p C(x y) -> PatchChoices p C(y z)
                       -> (FL p :> FL p) C(x z)
 selected_patches_last other_ps pc =
-  case get_choices pc of
-   fc :> mc :> lc -> other_ps +>+ mapFL_FL tp_patch (fc +>+ mc) :> mapFL_FL tp_patch lc
+  case getChoices pc of
+   fc :> mc :> lc -> other_ps +>+ mapFL_FL tpPatch (fc +>+ mc) :> mapFL_FL tpPatch lc
 
 selected_patches_first :: Patchy p => FL p C(y z) -> PatchChoices p C(x y)
                        -> (FL p :> FL p) C(x z)
 selected_patches_first other_ps pc =
-  case separate_first_from_middle_last pc of
-  xs :> ys -> mapFL_FL tp_patch xs :> mapFL_FL tp_patch ys +>+ other_ps
+  case separateFirstFromMiddleLast pc of
+  xs :> ys -> mapFL_FL tpPatch xs :> mapFL_FL tpPatch ys +>+ other_ps
 
 selected_patches_last_reversed :: Patchy p => FL p C(y x) -> PatchChoices p C(z y)
                                -> (FL p :> FL p) C(x z)
 selected_patches_last_reversed other_ps pc =
-  case separate_first_from_middle_last pc of
-  xs :> ys -> invert (mapFL_FL tp_patch ys +>+ other_ps) :> invert (mapFL_FL tp_patch xs)
+  case separateFirstFromMiddleLast pc of
+  xs :> ys -> invert (mapFL_FL tpPatch ys +>+ other_ps) :> invert (mapFL_FL tpPatch xs)
 
 selected_patches_first_reversed :: Patchy p => FL p C(z y) -> PatchChoices p C(y x)
                                 -> (FL p :> FL p) C(x z)
 selected_patches_first_reversed other_ps pc =
-  case get_choices pc of
-  fc :> mc :> lc -> invert (mapFL_FL tp_patch lc) :> invert (other_ps +>+ mapFL_FL tp_patch (fc +>+ mc))
+  case getChoices pc of
+  fc :> mc :> lc -> invert (mapFL_FL tpPatch lc) :> invert (other_ps +>+ mapFL_FL tpPatch (fc +>+ mc))
 
-text_select :: forall p C(x y z). Patchy p => String -> WhichChanges
+text_select :: forall p C(x y z). Patchy p => Maybe (Splitter p) -> String -> WhichChanges
             ->  MatchCriterion p -> [DarcsFlag] -> Int -> Int
             -> RL (TaggedPatch p) C(x y) -> FL (TaggedPatch p) C(y z) -> PatchChoices p C(x z)
             -> IO ((PatchChoices p) C(x z))
 
-text_select _ _ _ _ _ _ _ NilFL pc = return pc
-text_select jn whichch crit opts n_max n
+text_select _ _ _ _ _ _ _ _ NilFL pc = return pc
+text_select splitter jn whichch crit opts n_max n
             tps_done tps_todo@(tp:>:tps_todo') pc = do
       (printFriendly opts) `unseal2` viewp
       repeat_this -- prompt the user
     where
-        do_next_action ja je = tentatively_text_select ja jn je whichch crit opts
+        do_next_action ja je = tentatively_text_select splitter ja jn je whichch crit opts
                                           n_max
                                           (n+1) (tp:<:tps_done) tps_todo'
         do_next = do_next_action "" (Noun "patch")
@@ -414,6 +429,7 @@ text_select jn whichch crit opts n_max n
         helper = undefined
         thing  = Darcs.Patch.thing (helper pc)
         things = Darcs.Patch.things (helper pc)
+        split = splitter >>= flip applySplitter (tpPatch tp)
         options_basic =
            [ KeyPress 'y' (jn++" this "++thing)
            , KeyPress 'n' ("don't "++jn++" it")
@@ -434,7 +450,12 @@ text_select jn whichch crit opts n_max n
         options_nav =
            [ KeyPress 'j' ("skip to next "++thing)
            , KeyPress 'k' ("back up to previous "++thing) ]
+        options_split
+           | Just _ <- split
+                 = [ KeyPress 'e' ("interactively edit this "++thing) ]
+           | otherwise = []
         options = [options_basic]
+                  ++ [options_split]
                   ++ (if is_single_file_patch then [options_file] else [])
                   ++ [options_view ++
                       if Summary `elem` opts then [] else options_summary]
@@ -448,18 +469,28 @@ text_select jn whichch crit opts n_max n
           case yorn of
             'y' -> do_next $ force_yes (tag tp) pc
             'n' -> do_next $ force_no (tag tp) pc
-            'w' -> do_next $ make_uncertain (tag tp) pc
+            'w' -> do_next $ makeUncertain (tag tp) pc
+            'e' | Just (text, parse) <- split
+                -> do newText <- editText "darcs-patch-edit" text
+                      case parse newText of
+                        Nothing -> repeat_this
+                        Just ps -> do let tps_new = snd $ patchChoicesTpsSub (Just (tag tp)) ps
+                                      text_select splitter
+                                                  jn whichch crit opts (n_max + lengthFL tps_new - 1) n
+                                                  tps_done (tps_new+>+tps_todo')
+                                                  (substitute (seal2 (tp :||: tps_new)) pc)
+
             's' -> do_next_action "Skipped"  (Noun "change") $ skip_file
             'f' -> do_next_action "Included" (Noun "change") $ do_file
             'v' -> printPatch `unseal2` viewp >> repeat_this
             'p' -> printPatchPager `unseal2` viewp >> repeat_this
-            'l' -> do let selected = case get_choices pc of
+            'l' -> do let selected = case getChoices pc of
                                           (first_chs:>_:>last_chs) ->
                                              if whichch == Last || whichch == FirstReversed
                                                 then map_patches last_chs
                                                 else map_patches first_chs
                           map_patches = mapFL (\a ->
-                                           (showFriendly opts) `unseal2` (seal2 $ tp_patch a))
+                                           (showFriendly opts) `unseal2` (seal2 $ tpPatch a))
                       putStrLn $ "---- Already selected "++things++" ----"
                       mapM_ putDocLn $ selected
                       putStrLn $ "---- end of already selected "++things++" ----"
@@ -469,38 +500,38 @@ text_select jn whichch crit opts n_max n
                       repeat_this
             'd' -> return pc
             'a' -> do ask_confirmation
-                      return $ select_all_middles (whichch == Last || whichch == FirstReversed) pc
+                      return $ selectAllMiddles (whichch == Last || whichch == FirstReversed) pc
             'q' -> do putStrLn $ jn_cap++" cancelled."
                       exitWith $ ExitSuccess
             'j' -> case tps_todo' of
                        NilFL -> -- May as well work out the length now we have all
                                 -- the patches in memory
-                                text_select jn whichch crit opts
+                                text_select splitter jn whichch crit opts
                                     n_max n tps_done tps_todo pc
-                       _ -> text_select jn whichch crit opts
+                       _ -> text_select splitter jn whichch crit opts
                                 n_max (n+1) (tp:<:tps_done) tps_todo' pc
             'k' -> case tps_done of
                         NilRL -> repeat_this
                         (tp':<:tps_done') ->
-                           text_select jn whichch crit opts
+                           text_select splitter jn whichch crit opts
                                n_max (n-1) tps_done' (tp':>:tps_todo) pc
-            'c' -> text_select jn whichch crit opts
+            'c' -> text_select splitter jn whichch crit opts
                                         n_max n tps_done tps_todo pc
             _   -> do putStrLn $ helpFor jn options
                       repeat_this
-        force_yes = if whichch == Last || whichch == FirstReversed then force_last else force_first
-        force_no  = if whichch == Last || whichch == FirstReversed then force_first else force_last
+        force_yes = if whichch == Last || whichch == FirstReversed then forceLast else forceFirst
+        force_no  = if whichch == Last || whichch == FirstReversed then forceFirst else forceLast
         patches_to_skip = (tag tp:) $ catMaybes
-                        $ mapFL (\tp' -> if list_touched_files tp' == touched_files
+                        $ mapFL (\tp' -> if listTouchedFiles tp' == touched_files
                                          then Just (tag tp')
                                          else Nothing) tps_todo'
         skip_file = foldr force_no pc patches_to_skip
         do_file = foldr force_yes pc patches_to_skip
-        the_default = get_default (whichch == Last || whichch == FirstReversed) $ patch_slot tp pc
+        the_default = get_default (whichch == Last || whichch == FirstReversed) $ patchSlot tp pc
         jn_cap = (toUpper $ head jn) : tail jn
-        touched_files = list_touched_files $ tp_patch tp
+        touched_files = listTouchedFiles $ tpPatch tp
         is_single_file_patch = length touched_files == 1
-        viewp = if whichch == LastReversed || whichch == FirstReversed then seal2 $ invert (tp_patch tp) else seal2 $ tp_patch tp
+        viewp = if whichch == LastReversed || whichch == FirstReversed then seal2 $ invert (tpPatch tp) else seal2 $ tpPatch tp
         ask_confirmation =
             if jn `elem` ["unpull", "unrecord", "obliterate"]
             then do yorn <- askUser $ "Really " ++ jn ++ " all undecided patches? "
@@ -512,10 +543,10 @@ text_select jn whichch crit opts n_max n
 text_view :: forall p C(x y u r s). Patchy p => [DarcsFlag] -> Maybe Int -> Int
             -> RL (TaggedPatch p) C(x y) -> FL (TaggedPatch p) C(y u) -> PatchChoices p C(r s)
             -> IO ((PatchChoices p) C(r s))
-text_view _ _ _ _ NilFL _ = return $ patch_choices $ unsafeCoerceP NilFL --return pc
+text_view _ _ _ _ NilFL _ = return $ patchChoices $ unsafeCoerceP NilFL --return pc
 text_view opts n_max n
             tps_done tps_todo@(tp:>:tps_todo') pc = do
-      printFriendly opts (tp_patch tp)
+      printFriendly opts (tpPatch tp)
       putStr "\n"
       repeat_this -- prompt the user
     where
@@ -556,11 +587,11 @@ text_view opts n_max n
         repeat_this = do
           yorn <- promptCharFancy prompt (keysFor options) (Just 'n') "?h"
           case yorn of
-            'y' -> printPatch (tp_patch tp) >> next_patch
+            'y' -> printPatch (tpPatch tp) >> next_patch
             'n' -> next_patch
-            'v' -> printPatch (tp_patch tp) >> repeat_this
-            'p' -> printPatchPager (tp_patch tp) >> repeat_this
-            'x' -> do putDocLn $ prefix "    " $ summary (tp_patch tp)
+            'v' -> printPatch (tpPatch tp) >> repeat_this
+            'p' -> printPatchPager (tpPatch tp) >> repeat_this
+            'x' -> do putDocLn $ prefix "    " $ summary (tpPatch tp)
                       repeat_this
             'q' -> exitWith ExitSuccess
             'k' -> prev_patch
@@ -571,25 +602,25 @@ text_view opts n_max n
                       repeat_this
         count_n_max | isJust n_max = n_max
                     | otherwise    = Just $ lengthFL tps_todo + lengthRL tps_done
-tentatively_text_select :: Patchy p => String -> String -> Noun -> WhichChanges
+tentatively_text_select :: Patchy p => Maybe (Splitter p) -> String -> String -> Noun -> WhichChanges
                         -> MatchCriterion p -> [DarcsFlag]
                         -> Int -> Int -> RL (TaggedPatch p) C(x y) -> FL (TaggedPatch p) C(y z)
                         -> PatchChoices p C(x z)
                         -> IO ((PatchChoices p) C(x z))
-tentatively_text_select _ _ _ _ _ _ _ _ _ NilFL pc = return pc
-tentatively_text_select jobaction jobname jobelement whichch crit
+tentatively_text_select _ _ _ _ _ _ _ _ _ _ NilFL pc = return pc
+tentatively_text_select splitter jobaction jobname jobelement whichch crit
                         opts n_max n ps_done ps_todo pc =
-  case spanFL (\p -> decided $ patch_slot p pc) ps_todo of
+  case spanFL (\p -> decided $ patchSlot p pc) ps_todo of
   skipped :> unskipped -> do
    when (numSkipped > 0) show_skipped
    let (boringThenInteresting) =
                           if DontPromptForDependencies `elem` opts
-                          then spanFL (not.(crit whichch opts).tp_patch) unskipped
+                          then spanFL (not.(crit whichch opts).tpPatch) unskipped
                           else NilFL :> unskipped
    case boringThenInteresting of
      boring :> interesting -> do
      let numNotConsidered = lengthFL boring + numSkipped
-     text_select jobname whichch crit opts n_max (n + numNotConsidered)
+     text_select splitter jobname whichch crit opts n_max (n + numNotConsidered)
                  (reverseFL boring +<+ reverseFL skipped +<+ ps_done) interesting pc
    where
    numSkipped  = lengthFL skipped
@@ -601,7 +632,7 @@ tentatively_text_select jobaction jobname jobelement whichch crit
       _action_ = if (length jobaction) == 0 then "Skipped" else jobaction
       _elem_ = englishNum numSkipped jobelement
       showskippedpatch :: Patchy p => FL (TaggedPatch p) C(y t) -> IO ()
-      showskippedpatch (tp:>:tps) = (putDocLn $ prefix "    " $ summary (tp_patch tp)) >> showskippedpatch tps
+      showskippedpatch (tp:>:tps) = (putDocLn $ prefix "    " $ summary (tpPatch tp)) >> showskippedpatch tps
       showskippedpatch NilFL = return ()
 
 decided :: Slot -> Bool
@@ -617,4 +648,26 @@ get_default False InLast  = 'n'
 
 tp_patches :: (FL (TaggedPatch p) :> FL (TaggedPatch p)) C(x y)
            -> (FL p :> FL p) C(x y)
-tp_patches (x:>y) = mapFL_FL tp_patch x :> mapFL_FL tp_patch y
+tp_patches (x:>y) = mapFL_FL tpPatch x :> mapFL_FL tpPatch y
+
+-- |Optionally remove any patches (+dependencies) from a sequence that
+-- conflict with the recorded or unrecorded changes in a repo
+filterOutConflicts :: RepoPatch p
+                   => [DarcsFlag]                                    -- ^Command-line options. Only 'SkipConflicts' is
+                                                                     -- significant; filtering will happen iff it is present
+                   -> RL (PatchInfoAnd p) C(x r)                     -- ^Recorded patches from repository, starting from
+                                                                     -- same context as the patches to filter
+                   -> Repository p C(r u t)                          -- ^Repository itself, used for grabbing unrecorded changes
+                   -> FL (PatchInfoAnd p) C(x z)                     -- ^Patches to filter
+                   -> IO (Bool, Sealed (FL (PatchInfoAnd p) C(x)))   -- ^(True iff any patches were removed, possibly filtered patches)
+filterOutConflicts opts us repository them
+  | SkipConflicts `elem` opts
+     = do let commuter = commuterIdRL selfCommuter
+          unrec <- fmap n2pia . (anonymous . fromPrims) =<< unrecordedChanges [] repository []
+          them' :> rest <- return $ partitionConflictingFL commuter them (unrec :<: us)
+          return (check rest, Sealed them')
+  | otherwise
+     = return (False, Sealed them)
+  where check :: FL p C(a b) -> Bool
+        check NilFL = False
+        check _ = True
