@@ -30,7 +30,7 @@ module Darcs.SlurpDirectory.Internal
                         slurp_removefile, slurp_removedir,
                         slurp_remove,
                         slurp_modfile, slurp_hasfile, slurp_hasdir,
-                        slurp_has_anycase, wait_a_moment, undefined_time,
+                        slurp_has_anycase, undefined_time,
                         undefined_size,
                         slurp_has, list_slurpy, list_slurpy_files,
                         get_path_list,
@@ -56,7 +56,6 @@ import System.Posix.Files
           fileSize,
           isRegularFile, isDirectory, isSymbolicLink
         )
-import System.Posix ( sleep )
 import Data.Maybe ( catMaybes, isJust, maybeToList )
 import Data.Map (Map)
 import qualified Data.Map as Map
@@ -69,19 +68,11 @@ import ByteStringUtils
 import qualified Data.ByteString as B
 
 import Darcs.Patch.FileName ( FileName, fn2fp, fp2fn, norm_path, break_on_dir,
-                              own_name, super_name )
-#if mingw32_HOST_OS
-import Data.Int ( Int64 )
-#else
+                              own_name, superName )
 import System.Posix.Types ( FileOffset )
-#endif
 
 #include "impossible.h"
 
-#if mingw32_HOST_OS
-type FileOffset = Int64
-#endif
-
 data Slurpy = Slurpy !FileName !SlurpyContents
 
 slurpy_to_pair :: Slurpy -> (FileName, SlurpyContents)
@@ -183,7 +174,7 @@ write_file :: Slurpy -> FileName -> IO ()
 write_file s fn = case withSlurpy s $ smReadFilePS fn of
                      Left err -> fail err
                      Right (_, c) -> do
-                       ensureDirectories (super_name fn)
+                       ensureDirectories (superName fn)
                        mWriteFilePS fn c
                        
 try_write_file :: Slurpy -> FilePath -> IO ()
@@ -199,7 +190,7 @@ ensureDirectories d = do
           isPar <- mDoesDirectoryExist d
           if isPar 
             then return ()
-            else ensureDirectories (super_name d) >> (mCreateDirectory d)
+            else ensureDirectories (superName d) >> (mCreateDirectory d)
 
 write_files ::  Slurpy -> [FilePath] -> IO ()
 write_files s fps = mapM_ (try_write_file s) fps
@@ -256,7 +247,7 @@ modifyFileSlurpy f job = mksm sm
 
 insertSlurpy :: FileName -> Slurpy -> SlurpMonad ()
 insertSlurpy f news = mksm $ \s ->
-                      if slurp_hasfile f s || slurp_hasdir f s || not (slurp_hasdir (super_name f) s)
+                      if slurp_hasfile f s || slurp_hasdir f s || not (slurp_hasdir (superName f) s)
                       then Left $ "Error creating file "++fn2fp f
                       else Right (addslurp f news s, ())
 
@@ -341,13 +332,6 @@ undefined_size = -1
 undef_time_size :: (Maybe String, EpochTime, FileOffset)
 undef_time_size = (Nothing, undefined_time, undefined_size)
 
-wait_a_moment :: IO ()
-wait_a_moment = do { sleep 1; return () }
-    -- HACKERY: In ghc 6.1, sleep has the type signature IO Int; it
-    -- returns an integer just like sleep(3) does. To stay compatible
-    -- with older versions, though, we just ignore sleep's return
-    -- value. Hackery, like I said.
-
 isFileReallySymlink :: FilePath -> IO Bool
 isFileReallySymlink f = do fs <- getSymbolicLinkStatus f
                            return (isSymbolicLink fs)
@@ -536,7 +520,7 @@ slurp_removefile f s =
 
 slurp_move :: FileName -> FileName -> Slurpy -> Maybe Slurpy
 slurp_move f f' s =
-    if not (slurp_has (fn2fp f') s) && slurp_hasdir (super_name f') s
+    if not (slurp_has (fn2fp f') s) && slurp_hasdir (superName f') s
     then case get_slurp f s of
          Nothing -> Nothing
          Just sf ->
@@ -549,7 +533,7 @@ slurp_move f f' s =
 
 addslurp :: FileName -> Slurpy -> Slurpy -> Slurpy
 addslurp fname s s' =
-    case get_slurp_context (super_name fname) s' of
+    case get_slurp_context (superName fname) s' of
         Just (ctx, Slurpy d (SlurpDir _ c)) -> ctx (Slurpy d (SlurpDir Nothing (uncurry Map.insert (slurpy_to_pair s) c)))
         _ -> s'
 
@@ -567,7 +551,7 @@ slurp_removedir f s =
 
 slurp_adddir :: FileName -> Slurpy -> Maybe Slurpy
 slurp_adddir f s =
-  if slurp_hasfile f s || slurp_hasdir f s || not (slurp_hasdir (super_name f) s)
+  if slurp_hasfile f s || slurp_hasdir f s || not (slurp_hasdir (superName f) s)
   then Nothing
   else Just $ addslurp f (Slurpy (own_name f) (SlurpDir Nothing Map.empty)) s
 
