@@ -440,17 +440,10 @@ initM a = listArray (0, length a) (0:a)
 initP :: [B.ByteString] -> PArray
 initP a = listArray (0, length a) (B.empty:a)
 
-#if __GLASGOW_HASKELL__ > 604
 aLen :: (IArray a e) => a Int e -> Int
 aLen a = snd $ bounds a
 aLenM :: (MArray a e m) => a Int e -> m Int
 aLenM a = getBounds a >>= return . snd
-#else
-aLen :: HasBounds a => a Int e -> Int
-aLen a = snd $ bounds a
-aLenM :: (HasBounds a, Monad m) => a Int e -> m Int
-aLenM = return . snd . bounds
-#endif
 
 convertPatch :: Int -> PArray -> PArray -> (Int, Int, Int, Int)
              -> (Int,[B.ByteString],[B.ByteString])
