@@ -5,6 +5,7 @@ module Ssh ( grabSSH, runSSH, getSSH, copySSH, copySSHs, SSHCmd(..),
            ) where
 
 import Prelude hiding ( lookup, catch )
+import qualified Ratified( hGetContents )
 
 import System.Exit ( ExitCode(..) )
 import System.Environment ( getEnv )
@@ -15,7 +16,7 @@ import Darcs.Utils ( showHexLen )
 import Data.Bits ( (.&.) )
 import System.Random ( randomIO )
 #endif
-import System.IO ( Handle, hPutStr, hPutStrLn, hGetLine, hGetContents, hClose, hFlush )
+import System.IO ( Handle, hSetBinaryMode, hPutStr, hPutStrLn, hGetLine, hClose, hFlush )
 import System.IO.Unsafe ( unsafePerformIO )
 import System.Directory ( doesFileExist, createDirectoryIfMissing )
 import Control.Monad ( when )
@@ -42,20 +43,23 @@ sshConnections = unsafePerformIO $ newIORef empty
 
 data Connection = C { inp :: !Handle, out :: !Handle, err :: !Handle, deb :: String -> IO () }
 
-withSSHConnection :: String -> (Connection -> IO a) -> IO a -> IO a
-withSSHConnection x withconnection withoutconnection =
+withSSHConnection :: String -> String -> (Connection -> IO a) -> IO a -> IO a
+withSSHConnection rdarcs repoid withconnection withoutconnection =
     withoutProgress $
     do cs <- readIORef sshConnections
-       let uhost = takeWhile (/= ':') x
-           url = cleanrepourl x
+       let uhost = takeWhile (/= ':') repoid
+           url = cleanrepourl repoid
        case lookup url (cs :: Map String (Maybe Connection)) of
          Just Nothing -> withoutconnection
          Just (Just c) -> withconnection c
          Nothing ->
            do mc <- do (ssh,sshargs_) <- getSSHOnly SSH
-                       let sshargs = sshargs_ ++ [uhost,"darcs","transfer-mode","--repodir",cleanrepodir x]
+                       let sshargs = sshargs_ ++ [uhost,rdarcs,
+                                                  "transfer-mode","--repodir",cleanrepodir repoid]
                        debugMessage $ "ssh "++unwords sshargs
                        (i,o,e,_) <- runInteractiveProcess ssh sshargs Nothing Nothing
+                       hSetBinaryMode i True
+                       hSetBinaryMode o True
                        l <- hGetLine o
                        if l == "Hello user, I am darcs transfer mode"
                            then return ()
@@ -67,7 +71,7 @@ withSSHConnection x withconnection withoutconnection =
                     `catchNonSignal`
                             \e -> do debugMessage $ "Failed to start ssh connection:\n    "++
                                                     prettyException e
-                                     severSSHConnection x
+                                     severSSHConnection repoid
                                      debugMessage $ unlines $
                                          [ "NOTE: the server may be running a version of darcs prior to 2.0.0."
                                          , ""
@@ -98,10 +102,10 @@ grabSSH x c = do
                    clean "" = bug $ "Buggy path in grabSSH: "++x
                    file = clean dir
                    failwith e = do severSSHConnection x
-                                   eee <- hGetContents (err c) -- ratify hGetContents: it's okay
-                                                               -- here because we're only grabbing
-                                                               -- stderr, and we're also about to
-                                                               -- throw the contents.
+                                   -- hGetContents is ok here because we're
+                                   -- only grabbing stderr, and we're also
+                                   -- about to throw the contents.
+                                   eee <- Ratified.hGetContents (err c)
                                    debugFail $ e ++ " grabbing ssh file "++x++"\n"++eee
                deb c $ "get "++file
                hPutStrLn (inp c) $ "get " ++ file
@@ -123,8 +127,8 @@ sshStdErrMode :: IO Redirect
 sshStdErrMode = withDebugMode $ \amdebugging ->
                 return $ if amdebugging then AsIs else Null
 
-copySSH :: String -> FilePath -> IO ()
-copySSH uRaw f = withSSHConnection uRaw (\c -> grabSSH uRaw c >>= B.writeFile f) $
+copySSH :: String -> String -> FilePath -> IO ()
+copySSH rdarcs uRaw f = withSSHConnection rdarcs uRaw (\c -> grabSSH uRaw c >>= B.writeFile f) $
               do let u = escape_dollar uRaw
                  stderr_behavior <- sshStdErrMode
                  r <- runSSH SCP u [] [u,f] (AsIs,AsIs,stderr_behavior)
@@ -136,11 +140,11 @@ copySSH uRaw f = withSSHConnection uRaw (\c -> grabSSH uRaw c >>= B.writeFile f)
            where tr '$' = "\\$"
                  tr c = [c]
 
-copySSHs :: String -> [String] -> FilePath -> IO ()
-copySSHs u ns d =
-  withSSHConnection u (\c -> withCurrentDirectory d $
-                             mapM_ (\n -> grabSSH (u++"/"++n) c >>= B.writeFile n) $
-                             progressList "Copying via ssh" ns) $
+copySSHs :: String -> String -> [String] -> FilePath -> IO ()
+copySSHs rdarcs u ns d =
+  withSSHConnection rdarcs u (\c -> withCurrentDirectory d $
+                                mapM_ (\n -> grabSSH (u++"/"++n) c >>= B.writeFile n) $
+                                progressList "Copying via ssh" ns) $
   do let path = drop 1 $ dropWhile (/= ':') u
          host = takeWhile (/= ':') u
          cd = "cd "++path++"\n"
