@@ -5,16 +5,13 @@ module Workaround ( renameFile, setExecutable, getCurrentDirectory,
                     installHandler, raiseSignal, Handler(..), Signal,
                     sigINT, sigHUP, sigABRT, sigALRM, sigTERM, sigPIPE ) where
 
-#ifdef HAVE_SIGNALS
-import System.Posix.Signals(installHandler, raiseSignal, Handler(..), Signal,
-                         sigINT, sigHUP, sigABRT, sigALRM, sigTERM, sigPIPE,)
-#endif
-
 #ifdef WIN32
 import qualified System.Directory ( renameFile, getCurrentDirectory, removeFile )
 import qualified Control.Exception ( block )
 import qualified System.IO.Error ( isDoesNotExistError, ioError, catch )
 #else
+import System.Posix.Signals(installHandler, raiseSignal, Handler(..), Signal,
+                            sigINT, sigHUP, sigABRT, sigALRM, sigTERM, sigPIPE)
 import System.Directory ( renameFile, getCurrentDirectory )
 import System.Posix.Files (fileMode,getFileStatus, setFileMode,
                            setFileCreationMask,
@@ -24,7 +21,7 @@ import System.Posix.Files (fileMode,getFileStatus, setFileMode,
 import Data.Bits ( (.&.), (.|.), complement )
 #endif
 
-#ifndef HAVE_SIGNALS
+#ifdef WIN32
 -- Dummy implementation of POSIX signals
 data Handler = Default | Ignore | Catch (IO ())
 type Signal = Int
@@ -44,9 +41,7 @@ sigABRT = 0
 sigTERM = 0
 sigPIPE = 0
 sigALRM = 0
-#endif
 
-#ifdef WIN32
 {-
 System.Directory.renameFile incorrectly fails when the new file already
 exists.  This code works around that bug at the cost of losing atomic
