@@ -1,18 +1,17 @@
-#!/bin/sh
+#!/usr/bin/env bash
 set -ev
 
-test $DARCS || DARCS=$PWD/../darcs
 rm -rf temp1 temp2
 mkdir temp1 temp2
 cd temp1
-$DARCS init
+darcs init
 echo zig > foo
-$DARCS add foo
+darcs add foo
 #sleep 1
-$DARCS record -a -m add_foo -A x
+darcs record -a -m add_foo -A x
 cd ../temp2
-$DARCS init
-$DARCS pull -a ../temp1
+darcs init
+darcs pull -a ../temp1
 cd ..
 cmp temp1/foo temp2/foo
 rm -rf temp1 temp2
