@@ -2,6 +2,9 @@
 
 # A test for unrecording checkpoint tags, inspired by issue517
 
+exit 200 # checkpoint creation is not supported by current darcs
+         # we would need a testdata tarball for this test
+
 set -ev
 
 darcs --version
