@@ -10,10 +10,9 @@ touch t.t
 darcs add t.t
 darcs record -am "initial add"
 darcs changes --context > my_context
-IFS='' # annoying hack for cygwin and hspwd below
-DIR=`hspwd`
-abs_to_context=${DIR}/my_context
+DIR=`pwd`
+abs_to_context="${DIR}/my_context"
 cd ..
 rm -rf temp2
-darcs get temp1 --context=${abs_to_context} temp2
+darcs get temp1 --context="${abs_to_context}" temp2
 rm -rf temp1 temp2
