@@ -1,19 +1,17 @@
-#!/bin/sh
+#!/usr/bin/env bash
 set -ev
 
-test $DARCS || DARCS=$PWD/../darcs
-
 rm -rf temp
 mkdir temp
 cd temp
 
-$DARCS initialize
+darcs initialize
 echo text > afile.txt
-$DARCS add afile.txt
-$DARCS record --author me --all --no-test --patch-name init
+darcs add afile.txt
+darcs record --author me --all --no-test --patch-name init
 mkdir d
 echo The following mv should fail, since d isnt in the repo.
-if $DARCS mv afile.txt d/afile.txt; then
+if darcs mv afile.txt d/afile.txt; then
 false
 fi
 
