@@ -14,7 +14,7 @@ darcs annotate -p .
 echo newtext > foo
 darcs record -am 'modify foo'
 
-darcs diff --store --last=1 > out1
+darcs diff --store-in-mem --last=1 > out1
 cat out1
 grep text out1
 grep foo out1
