@@ -1,5 +1,8 @@
 #!/usr/bin/env bash
 
+exit 200 # checkpoint creation is not supported by current darcs
+         # we would need a testdata tarball for this test
+
 # A partial get of a repo shall have the same _darcs/pristine as the original
 
 set -ev
