@@ -22,8 +22,7 @@ EOF
 write_buggy_pending
 
 echo now watch the fireworks as all sorts of things fail
-not darcs whatsnew &> out
-cat out
+not darcs whatsnew 2>&1 | tee out
 grep 'pending has conflicts' out
 
 echo pending should now be fixed but there are no changes
@@ -31,8 +30,7 @@ not darcs whatsnew
 
 write_buggy_pending
 
-darcs revert -a &> out
-cat out
+darcs revert -a 2>&1 | tee out
 grep 'pending has conflicts' out
 
 echo pending should now be emptied
@@ -40,22 +38,19 @@ darcs revert -a
 
 write_buggy_pending
 
-darcs record -a -m foo &> out
-cat out
+darcs record -a -m foo 2>&1 | tee out
 grep 'pending has conflicts' out
 
 darcs record -a -m foo
 
 darcs changes -v
 
-darcs repair &> out
-cat out
+darcs repair 2>&1 | tee out
 grep 'The repository is already consistent' out
 
 write_buggy_pending
 
-darcs repair &> out
-cat out
+darcs repair 2>&1 | tee out
 grep 'The repository is already consistent' out
 
 cd ..
