@@ -10,7 +10,7 @@ rm -rf temp1 temp2
 mkdir temp1
 cd temp1
 darcs init
-perl -e 'print "a"x1048576' > foo
+printf "%01048576d" 0 > foo
 darcs record -l -a -A author -m xx
 rm foo
 darcs record -a -A author -m yy
