@@ -26,7 +26,8 @@ darcs get temp1 temp4
 
 chgrec () {
     set -ev
-    perl -i~ -pe "$1" foo
+    sed -e "$1" foo > foo.tmp
+    mv foo.tmp foo
     darcs record -a --ignore-times -m "$2"
 }
 
