@@ -4,6 +4,14 @@
 
 . lib
 
+slash() {
+if echo $OS | grep -q -i windows; then
+    echo -n \\
+else
+    echo -n /
+fi
+}
+
 DIR="`pwd`"
 
 rm -rf temp1 temp2
@@ -40,7 +48,7 @@ cd temp1
 # Before trying to pull from self, defaultrepo does not exist
 test ! -e _darcs/prefs/defaultrepo
 # return special message when you try to push to yourself
-not darcs push -a "$DIR/temp1" 2> log
+not darcs push -a "$DIR`slash`temp1" 2> log
 grep -i "cannot push from repository to itself" log
 # and don't update the default repo to be the current dir
 test ! -e _darcs/prefs/defaultrepo
