@@ -21,11 +21,9 @@ cd ..
 # put to self
 cd temp1
 not grep temp1 _darcs/prefs/defaultrepo
-# hack for cygwin and hspwd below
-IFS=''
-DIR=`hspwd`
+DIR=`pwd`
 # return special message when you try to put put yourself
-not darcs put ${DIR} 2> log
+not darcs put "$DIR" 2> log
 not grep temp1 _darcs/prefs/defaultrepo
 grep -i "Can't put.*current repository" log
 cd ..
