@@ -2,7 +2,6 @@
 set -ev
 
 check_manifest () {
-    IFS=' '
     : > files.tmp
     echo . > dirs.tmp
     echo . > files-dirs.tmp
@@ -18,11 +17,9 @@ check_manifest () {
     darcs query manifest $3 --no-files --directories | sort > darcs-dirs.tmp
     darcs query manifest $3 --files --directories | sort > darcs-files-dirs.tmp
     for x in files dirs files-dirs ; do
-        sort -o sorted-$x.tmp $x.tmp
+        sort $x.tmp >sorted-$x.tmp
         diff sorted-$x.tmp darcs-$x.tmp
-    done
-    IFS=''
-}
+    done }
 
 rm -rf temp
 mkdir temp
