@@ -15,7 +15,7 @@ cd temp1
 darcs init
 touch file.t
 darcs add file.t
-darcs record --edit-long-comment -a -m foo file.t | grep '2.*END OF DESCRIPTION'
+echo y | darcs record --edit-long-comment -a -m foo file.t | grep '2.*END OF DESCRIPTION'
 cd ..
 rm -rf temp1
 
@@ -25,7 +25,7 @@ cd temp2\ dir
 darcs init
 touch file.t
 darcs add file.t
-darcs record --edit-long-comment -a -m foo file.t | grep '2.*END OF DESCRIPTION'
+echo y | darcs record --edit-long-comment -a -m foo file.t | grep '2.*END OF DESCRIPTION'
 cd ..
 rm -rf temp2\ dir
 
@@ -36,7 +36,7 @@ cd temp1
 darcs init
 touch file.t
 darcs add file.t
-darcs record --edit-long-comment -a -m foo file.t | grep 'END OF'
+echo y | darcs record --edit-long-comment -a -m foo file.t | grep 'END OF'
 cd ..
 rm -rf temp1
 
@@ -53,7 +53,7 @@ echo > test-command << FOO
 echo EVIL
 FOO
 chmod u+x test-command
-darcs record --logfile='; test-command' --edit-long-comment -a -m foo file.t > log
+echo y | darcs record --logfile='; test-command' --edit-long-comment -a -m foo file.t > log
 not grep EVIL log
 cd ..
 rm -rf temp1
