@@ -1,9 +1,7 @@
 #!/usr/bin/env bash
-set -ev
 
 ## All these commands SHOULD fail (hence leading NOTs).
-## I would use the builtin !, but that has the wrong semantics.
-not () { "$@" && exit 1 || :; }
+. lib
 
 rm -rf test
 mkdir test
