@@ -1,5 +1,5 @@
 #!/usr/bin/env bash
-set -ev
+. lib
 
 rm -rf temp1
 mkdir temp1
@@ -21,6 +21,8 @@ darcs show contents foo | grep fourth
 darcs show contents foo -p third | grep third
 darcs show contents foo --match="author author1" first | grep first
 darcs show contents foo --tag t1 | grep second
+not darcs show contents foo --match "hash bla" 2>&1 | tee out
+grep "Couldn't match pattern" out
 cd ..
 
 rm -rf temp1
