@@ -2,6 +2,9 @@
 
 set -ev
 
+exit 200 # checkpoint creation is not supported by current darcs
+         # we would need a testdata tarball for this test
+
 # Check that checkpoints are removed when tags are unrecorded
 
 rm -rf temp1
